from typing import Optional

from highcharts_core.decorators import class_sensitive
from highcharts_core.metaclasses import HighchartsMeta


class LegendKeyboardNavigation(HighchartsMeta):
    """Options for keyboard navigation of the legend."""

    def __init__(self, **kwargs):
        self._enabled = None

        self.enabled = kwargs.get('enabled', None)

    @property
    def _dot_path(self) -> Optional[str]:
        """The dot-notation path to the options key for the current class.
        
        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return 'legend.accessibility.keyboardNavigation'

    @property
    def enabled(self) -> Optional[bool]:
        """If ``True``, enables keyboard navigation for the legend. Defaults to ``True``.

        :rtype: :class:`bool <python:bool>` or :obj:`None <python:None>`
        """
        return self._enabled

    @enabled.setter
    def enabled(self, value):
        if value is None:
            self._enabled = None
        else:
            self._enabled = bool(value)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'enabled': as_dict.get('enabled', None)
        }
        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        return {
            'enabled': self.enabled
        }


class LegendAccessibilityOptions(HighchartsMeta):
    """Accessibility options for the legend.

    .. note::

      Requires the Accessibility module.

    """

    def __init__(self, **kwargs):
        self._enabled = None
        self._keyboard_navigation = None

        self.enabled = kwargs.get('enabled', None)
        self.keyboard_navigation = kwargs.get('keyboard_navigation', None)

    @property
    def _dot_path(self) -> Optional[str]:
        """The dot-notation path to the options key for the current class.
        
        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return 'legend.accessibility'

    @property
    def enabled(self) -> Optional[bool]:
        """If ``True``, enables accessibility support for the legend. Defaults to
        ``True``.

        :rtype: :class:`bool <python:bool>` or :obj:`None <python:None>`
        """
        return self._enabled

    @enabled.setter
    def enabled(self, value):
        if value is None:
            self._enabled = None
        else:
            self._enabled = bool(value)

    @property
    def keyboard_navigation(self) -> Optional[LegendKeyboardNavigation]:
        """Options for keyboard navigation of the legend.

        :rtype: :class:`LegendKeyboardNavigation` or :obj:`None <python:None>`
        """
        return self._keyboard_navigation

    @keyboard_navigation.setter
    @class_sensitive(LegendKeyboardNavigation)
    def keyboard_navigation(self, value):
        self._keyboard_navigation = value

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'enabled': as_dict.get('enabled', None),
            'keyboard_navigation': as_dict.get('keyboardNavigation', None)
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'enabled': self.enabled,
            'keyboardNavigation': self.keyboard_navigation
        }

        return untrimmed



"""Tests for ``highcharts.no_data``."""

import pytest

from json.decoder import JSONDecodeError

from highcharts_core.global_options.language.navigation import NavigationLanguageOptions as cls
from highcharts_core.global_options.language.navigation import PopupLanguageOptions as cls2
from highcharts_core import errors
from tests.fixtures import input_files, check_input_file, to_camelCase, to_js_dict, \
    Class__init__, Class__to_untrimmed_dict, Class_from_dict, Class_to_dict, \
    Class_from_js_literal

STANDARD_PARAMS = [
    ({}, None),
    ({
      'popup': {
        'addButton': 'Add',
        'algorithm': 'some string',
        'arrowInfinityLine': 'some string',
        'arrowRay': 'some string',
        'arrowSegment': 'string',
        'average': 'avg',
        'background': 'background',
        'backgroundColor': 'some string',
        'borderRadius': 'some string',
        'borderWidth': 'width',
        'bottomBand': 'band',
        'circle': 'some string',
        'clearFilter': 'clear',
        'color': 'some string',
        'connector': 'some string',
        'crooked3': 'some string',
        'crooked5': 'some string',
        'crosshairX': 'some string',
        'crosshairY': 'some string',
        'decimals': 'some string',
        'deviation': 'some string',
        'editButton': 'some string',
        'elliott3': 'some string',
        'elliott5': 'some string',
        'ellipse': 'some string',
        'factor': 'some string',
        'fastAvgPeriod': 'some string',
        'fibonacci': 'some string',
        'fibonacciTimeZones': 'some string',
        'fill': 'some string',
        'flags': 'some string',
        'fontSize': 'some string',
        'format': 'some string',
        'height': 'some string',
        'highIndex': 'some string',
        'horizontalLine': 'some string',
        'increment': 'some string',
        'index': 'some string',
        'infinityLine': 'some string',
        'initialAccelerationFactor': 'some string',
        'innerBackground': 'some string',
        'label': 'some string',
        'labelOptions': 'some string',
        'labels': 'some string',
        'line': 'some string',
        'lines': 'some string',
        'longPeriod': 'some string',
        'lowIndex': 'some string',
        'maxAccelerationFactor': 'some string',
        'measure': 'some string',
        'measureX': 'some string',
        'measureXY': 'some string',
        'measureY': 'some string',
        'multiplier': 'some string',
        'multiplierATR': 'some string',
        'name': 'some string',
        'noFilterMatch': 'some string',
        'outerBackground': 'some string',
        'padding': 'some string',
        'parallelChannel': 'some string',
        'period': 'some string',
        'periodATR': 'some string',
        'periods': 'some string',
        'periodSenkouSpanB': 'some string',
        'periodTenkan': 'some string',
        'pitchfork': 'some string',
        'ranges': 'some string',
        'ray': 'some string',
        'rectangle': 'some string',
        'removeButton': 'some string',
        'saveButton': 'some string',
        'searchIndicators': 'some string',
        'segment': 'some string',
        'series': 'some string',
        'shapeOptions': 'some string',
        'shapes': 'some string',
        'shortPeriod': 'some string',
        'signalPeriod': 'some string',
        'simpleShapes': 'some string',
        'slowAvgPeriod': 'some string',
        'standardDeviation': 'some string',
        'stroke': 'some string',
        'strokeWidth': 'some string',
        'style': 'some string',
        'timeCycles': 'some string',
        'title': 'some string',
        'topBand': 'some string',
        'tunnel': 'some string',
        'typeOptions': 'some string',
        'verticalArrow': 'some string',
        'verticalCounter': 'some string',
        'verticalLabel': 'some string',
        'verticalLine': 'some string',
        'volume': 'some string',
        'xAxisUnit': 'some string'
      }
    }, None),
]


@pytest.mark.parametrize('kwargs, error', STANDARD_PARAMS)
def test__init__(kwargs, error):
    Class__init__(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error', STANDARD_PARAMS)
def test__to_untrimmed_dict(kwargs, error):
    Class__to_untrimmed_dict(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error',  STANDARD_PARAMS)
def test_from_dict(kwargs, error):
    Class_from_dict(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error',  STANDARD_PARAMS)
def test_to_dict(kwargs, error):
    Class_to_dict(cls, kwargs, error)


@pytest.mark.parametrize('filename, as_file, error', [
    ('global_options/language/navigation/01.js', False, None),

    ('global_options/language/navigation/error-01.js', False, (errors.HighchartsValueError,
                                                               errors.HighchartsParseError,
                                                               JSONDecodeError,
                                                               TypeError,
                                                               ValueError)),

    ('global_options/language/navigation/01.js', True, None),

    ('global_options/language/navigation/error-01.js', True, (errors.HighchartsValueError,
                                                              errors.HighchartsParseError,
                                                              JSONDecodeError,
                                                              TypeError,
                                                              ValueError)),
])
def test_from_js_literal(input_files, filename, as_file, error):
    Class_from_js_literal(cls, input_files, filename, as_file, error)


STANDARD_PARAMS_2 = [
    ({}, None),
    ({
      'add_button': 'Add',
      'algorithm': 'some string',
      'arrow_infinity_line': 'some string',
      'arrow_ray': 'some string',
      'arrow_segment': 'string',
      'average': 'avg',
      'background': 'background',
      'background_color': 'some string',
      'border_radius': 'some string',
      'border_width': 'width',
      'bottom_band': 'band',
      'circle': 'some string',
      'clear_filter': 'clear',
      'color': 'some string',
      'connector': 'some string',
      'crooked3': 'some string',
      'crooked5': 'some string',
      'crosshairX': 'some string',
      'crosshairY': 'some string',
      'decimals': 'some string',
      'deviation': 'some string',
      'edit_button': 'some string',
      'elliott3': 'some string',
      'elliott5': 'some string',
      'ellipse': 'some string',
      'factor': 'some string',
      'fast_avg_period': 'some string',
      'fibonacci': 'some string',
      'fibonacci_time_zones': 'some string',
      'fill': 'some string',
      'flags': 'some string',
      'font_size': 'some string',
      'format': 'some string',
      'height': 'some string',
      'high_index': 'some string',
      'horizontal_line': 'some string',
      'increment': 'some string',
      'index': 'some string',
      'infinity_line': 'some string',
      'initial_acceleration_factor': 'some string',
      'inner_background': 'some string',
      'label': 'some string',
      'label_options': 'some string',
      'labels': 'some string',
      'line': 'some string',
      'lines': 'some string',
      'long_period': 'some string',
      'low_index': 'some string',
      'max_acceleration_factor': 'some string',
      'measure': 'some string',
      'measure_x': 'some string',
      'measure_xy': 'some string',
      'measure_y': 'some string',
      'multiplier': 'some string',
      'multiplier_atr': 'some string',
      'name': 'some string',
      'no_filter_match': 'some string',
      'outer_background': 'some string',
      'padding': 'some string',
      'parallel_channel': 'some string',
      'period': 'some string',
      'period_atr': 'some string',
      'periods': 'some string',
      'period_senkou_span_b': 'some string',
      'period_tenkan': 'some string',
      'pitchfork': 'some string',
      'ranges': 'some string',
      'ray': 'some string',
      'rectangle': 'some string',
      'remove_button': 'some string',
      'save_button': 'some string',
      'search_indicators': 'some string',
      'segment': 'some string',
      'series': 'some string',
      'shape_options': 'some string',
      'shapes': 'some string',
      'short_period': 'some string',
      'signal_period': 'some string',
      'simple_shapes': 'some string',
      'slow_avg_period': 'some string',
      'standard_deviation': 'some string',
      'stroke': 'some string',
      'stroke_width': 'some string',
      'style': 'some string',
      'time_cycles': 'some string',
      'title': 'some string',
      'top_band': 'some string',
      'tunnel': 'some string',
      'type_options': 'some string',
      'vertical_arrow': 'some string',
      'vertical_counter': 'some string',
      'vertical_label': 'some string',
      'vertical_line': 'some string',
      'volume': 'some string',
      'x_axis_unit': 'some string'
    }, None),
]


@pytest.mark.parametrize('kwargs, error', STANDARD_PARAMS_2)
def test_PopupLanguageOptions__init__(kwargs, error):
    Class__init__(cls2, kwargs, error)


@pytest.mark.parametrize('kwargs, error', STANDARD_PARAMS_2)
def test_PopupLanguageOptions__to_untrimmed_dict(kwargs, error):
    Class__to_untrimmed_dict(cls2, kwargs, error)


@pytest.mark.parametrize('kwargs, error',  STANDARD_PARAMS_2)
def test_PopupLanguageOptions_from_dict(kwargs, error):
    Class_from_dict(cls2, kwargs, error)


@pytest.mark.parametrize('kwargs, error',  STANDARD_PARAMS_2)
def test_PopupLanguageOptions_to_dict(kwargs, error):
    Class_to_dict(cls2, kwargs, error)


@pytest.mark.parametrize('filename, as_file, error', [
    ('global_options/language/navigation/02.js', False, None),

    ('global_options/language/navigation/error-03.js', False, (errors.HighchartsValueError,
                                                               errors.HighchartsParseError,
                                                               JSONDecodeError,
                                                               TypeError,
                                                               ValueError)),

    ('global_options/language/navigation/02.js', True, None),

    ('global_options/language/navigation/error-03.js', True, (errors.HighchartsValueError,
                                                              errors.HighchartsParseError,
                                                              JSONDecodeError,
                                                              TypeError,
                                                              ValueError)),
])
def test_PopupLanguageOptions_from_js_literal(input_files, filename, as_file, error):
    Class_from_js_literal(cls2, input_files, filename, as_file, error)



from typing import Optional
from decimal import Decimal

from validator_collection import validators

from highcharts_core import errors
from highcharts_core.decorators import class_sensitive
from highcharts_core.metaclasses import HighchartsMeta
from highcharts_core.options.plot_options.networkgraph import NetworkGraphOptions


class ParentNodeOptions(HighchartsMeta):
    """Series options for parent nodes."""

    def __init__(self, **kwargs):
        self._allow_point_select = None

        self.allow_point_select = kwargs.get('allow_point_select', None)

    @property
    def allow_point_select(self) -> Optional[bool]:
        """Allow this series' parent nodes to be selected by clicking on the graph.
        Defaults to ``False``.

        :rtype: :class:`bool <python:None>` or :obj:`None <python:None>`
        """
        return self._allow_point_select

    @allow_point_select.setter
    def allow_point_select(self, value):
        if value is None:
            self._allow_point_select = None
        else:
            self._allow_point_select = bool(value)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        return {
            'allow_point_select': as_dict.get('allowPointSelect', None)
        }

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        return {
            'allowPointSelect': self.allow_point_select
        }


class PackedBubbleOptions(NetworkGraphOptions):
    """General options to apply to all Packed Bubble series types.

    A packed bubble series is a two dimensional series type, where each point renders
    a value in X, Y position. Each point is drawn as a bubble where the bubbles don't
    overlap with each other and the radius of the bubble relates to the value.

    .. tabs::

      .. tab:: Standard Packed Bubble

        .. figure:: ../../../_static/packedbubble-example.png
          :alt: Split Packed Bubble Example Chart
          :align: center

      .. tab:: Split Packed Bubble

        .. figure:: ../../../_static/packedbubble-example-split.png
          :alt: Split Packed Bubble Example Chart
          :align: center

    """

    def __init__(self, **kwargs):
        self._display_negative = None
        self._max_size = None
        self._min_size = None
        self._parent_node = None
        self._size_by = None
        self._use_simulation = None
        self._z_threshold = None

        self.display_negative = kwargs.get('display_negative', None)
        self.max_size = kwargs.get('max_size', None)
        self.min_size = kwargs.get('min_size', None)
        self.parent_node = kwargs.get('parent_node', None)
        self.size_by = kwargs.get('size_by', None)
        self.use_simulation = kwargs.get('use_simulation', None)
        self.z_threshold = kwargs.get('z_threshold', None)

        super().__init__(**kwargs)

    @property
    def display_negative(self) -> Optional[bool]:
        """If ``True``, display negative sized bubbles.

        The threshold is given by the :meth:`z_threshold <BubbleOptions.z_threshold>`
        setting, and negative bubbles can be visualized by setting
        :meth:`negative_color <BubbleOptions.negative_color>`.

        :rtype: :class:`bool <python:bool>` or :obj:`None <python:None>`
        """
        return self._display_negative

    @display_negative.setter
    def display_negative(self, value):
        if value is None:
            self._display_negative = None
        else:
            self._display_negative = bool(value)

    @property
    def max_size(self) -> Optional[str | int | float | Decimal]:
        """Maximum bubble size. Defaults to ``'20%'``.

        If :obj:`None <python:None>`, bubbles will automatically size between the
        :meth:`min_size <BubbleOptions.min_size>` and ``max_size``, to reflect the z value
        of each bubble. Can be either pixels (when no unit is given), or a percentage of
        the smallest one of the plot width and height.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._max_size

    @max_size.setter
    def max_size(self, value):
        if value is None:
            self._max_size = None
        else:
            try:
                value = validators.string(value)
                if '%' not in value:
                    raise errors.HighchartsValueError('max_size expects either a number '
                                                      'or a % string. No "%" found.')
            except TypeError:
                self._max_size = validators.numeric(value,
                                                    allow_empty = True,
                                                    minimum = 0)

    @property
    def min_size(self) -> Optional[str | int | float | Decimal]:
        """Minimum bubble size. Defaults to ``8``.

        If :obj:`None <python:None>`, bubbles will automatically size between the
        :meth:`max_size <BubbleOptions.max_size>` and ``min_size``, to reflect the z value
        of each bubble. Can be either pixels (when no unit is given), or a percentage of
        the smallest one of the plot width and height.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._min_size

    @min_size.setter
    def min_size(self, value):
        if value is None:
            self._min_size = None
        else:
            try:
                value = validators.string(value)
                if '%' not in value:
                    raise errors.HighchartsValueError('min_size expects either a number '
                                                      'or a % string. No "%" found.')
            except TypeError:
                self._min_size = validators.numeric(value,
                                                    allow_empty = True,
                                                    minimum = 0)

    @property
    def parent_node(self) -> Optional[ParentNodeOptions]:
        """Series options for parent nodes.

        :rtype: :class:`ParentNodeOptions` or :obj:`None <python:None>`
        """
        return self._parent_node

    @parent_node.setter
    @class_sensitive(ParentNodeOptions)
    def parent_node(self, value):
        self._parent_node = value

    @property
    def size_by(self) -> Optional[str]:
        """Whether the bubble's value should be represented by the ``'area'`` or the
        ``'width'`` of the bubble. The default, ``'area'``, corresponds best to the human
        perception of the size of each bubble.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._size_by

    @size_by.setter
    def size_by(self, value):
        if not value:
            self._size_by = None
        else:
            value = validators.string(value)
            value = value.lower()
            if value not in ['area', 'width']:
                raise errors.HighchartsValueError(f'size_by expects either "area", '
                                                  f'or "width". Received: {value}')
            self._size_by = value

    @property
    def use_simulation(self) -> Optional[bool]:
        """If ``True``, simulation is used to calculate bubble positions. Defaults to
        ``True``.

        Simulation also adds options to the series graph based on used layout. In case of
        big data sets, with any performance issues, it is possible to disable animation
        and pack bubble in a simple circular way.

        :rtype: :class:`bool <python:bool>` or :obj:`None <python:None>`
        """
        return self._use_simulation

    @use_simulation.setter
    def use_simulation(self, value):
        if value is None:
            self._use_simulation = None
        else:
            self._use_simulation = bool(value)

    @property
    def z_threshold(self) -> Optional[int | float | Decimal]:
        """When :meth:`display_negative <BubbleOptions.display_negative>` is ``False``,
        then bubbles with a Z value lower than ``z_threshold`` are not rendered. When
        :meth:`display_negative <BubbleOptions.display_negative>` is ``True`` and
        :meth:`negative_color <BubbleOptions.negative_color>` is set, then points
        with a Z value lower than ``z_threshold`` are rendered with the negative coloring.

        Defaults to ``0``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._z_threshold

    @z_threshold.setter
    def z_threshold(self, value):
        self._z_threshold = validators.numeric(value, allow_empty = True)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'accessibility': as_dict.get('accessibility', None),
            'allow_point_select': as_dict.get('allowPointSelect', None),
            'animation': as_dict.get('animation', None),
            'class_name': as_dict.get('className', None),
            'clip': as_dict.get('clip', None),
            'color': as_dict.get('color', None),
            'cursor': as_dict.get('cursor', None),
            'custom': as_dict.get('custom', None),
            'dash_style': as_dict.get('dashStyle', None),
            'data_labels': as_dict.get('dataLabels', None),
            'description': as_dict.get('description', None),
            'enable_mouse_tracking': as_dict.get('enableMouseTracking', None),
            'events': as_dict.get('events', None),
            'include_in_data_export': as_dict.get('includeInDataExport', None),
            'keys': as_dict.get('keys', None),
            'label': as_dict.get('label', None),
            'legend_symbol': as_dict.get('legendSymbol', None),
            'linked_to': as_dict.get('linkedTo', None),
            'marker': as_dict.get('marker', None),
            'on_point': as_dict.get('onPoint', None),
            'opacity': as_dict.get('opacity', None),
            'point': as_dict.get('point', None),
            'point_description_formatter': as_dict.get('pointDescriptionFormatter', None),
            'selected': as_dict.get('selected', None),
            'show_checkbox': as_dict.get('showCheckbox', None),
            'show_in_legend': as_dict.get('showInLegend', None),
            'skip_keyboard_navigation': as_dict.get('skipKeyboardNavigation', None),
            'sonification': as_dict.get('sonification', None),
            'states': as_dict.get('states', None),
            'sticky_tracking': as_dict.get('stickyTracking', None),
            'threshold': as_dict.get('threshold', None),
            'tooltip': as_dict.get('tooltip', None),
            'turbo_threshold': as_dict.get('turboThreshold', None),
            'visible': as_dict.get('visible', None),

            'animation_limit': as_dict.get('animationLimit', None),
            'boost_blending': as_dict.get('boostBlending', None),
            'boost_threshold': as_dict.get('boostThreshold', None),
            'color_axis': as_dict.get('colorAxis', None),
            'color_key': as_dict.get('colorKey', None),
            'connect_ends': as_dict.get('connectEnds', None),
            'connect_nulls': as_dict.get('connectNulls', None),
            'crop_threshold': as_dict.get('cropThreshold', None),
            'data_sorting': as_dict.get('dataSorting', None),
            'drag_drop': as_dict.get('dragDrop', None),
            'fill_color': as_dict.get('fillColor', None),
            'fill_opacity': as_dict.get('fillOpacity', None),
            'get_extremes_from_all': as_dict.get('getExtremesFromAll', None),
            'linecap': as_dict.get('linecap', None),
            'line_color': as_dict.get('lineColor', None),
            'negative_color': as_dict.get('negativeColor', None),
            'negative_fill_color': as_dict.get('negativeFillColor', None),
            'point_interval': as_dict.get('pointInterval', None),
            'point_interval_unit': as_dict.get('pointIntervalUnit', None),
            'point_placement': as_dict.get('pointPlacement', None),
            'point_start': as_dict.get('pointStart', None),
            'soft_threshold': as_dict.get('softThreshold', None),
            'stacking': as_dict.get('stacking', None),
            'step': as_dict.get('step', None),
            'track_by_area': as_dict.get('trackByArea', None),
            'zone_axis': as_dict.get('zoneAxis', None),

            'color_index': as_dict.get('colorIndex', None),
            'crisp': as_dict.get('crisp', None),
            'draggable': as_dict.get('draggable', None),
            'find_nearest_point_by': as_dict.get('findNearestPointBy', None),
            'layout_algorithm': as_dict.get('layoutAlgorithm', None),
            'line_width': as_dict.get('lineWidth', None),
            'link': as_dict.get('link', None),
            'relative_x_value': as_dict.get('relativeXValue', None),
            'shadow': as_dict.get('shadow', None),
            'zones': as_dict.get('zones', None),

            'display_negative': as_dict.get('displayNegative', None),
            'max_size': as_dict.get('maxSize', None),
            'min_size': as_dict.get('minSize', None),
            'parent_node': as_dict.get('parentNode', None),
            'size_by': as_dict.get('sizeBy', None),
            'use_simulation': as_dict.get('useSimulation', None),
            'z_threshold': as_dict.get('zThreshold', None)
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'displayNegative': self.display_negative,
            'maxSize': self.max_size,
            'minSize': self.min_size,
            'parentNode': self.parent_node,
            'sizeBy': self.size_by,
            'useSimulation': self.use_simulation,
            'zThreshold': self.z_threshold
        }
        parent_as_dict = super()._to_untrimmed_dict(in_cls = in_cls)

        for key in parent_as_dict:
            untrimmed[key] = parent_as_dict[key]

        return untrimmed



from typing import Optional, List

from validator_collection import validators

from highcharts_core import constants
from highcharts_core.decorators import validate_types, class_sensitive
from highcharts_core.metaclasses import HighchartsMeta
from highcharts_core.utility_classes.animation import AnimationOptions
from highcharts_core.utility_classes.breadcrumbs import BreadcrumbOptions
from highcharts_core.options.series.base import SeriesBase
from highcharts_core.options.series.series_generator import create_series_obj


class Drilldown(HighchartsMeta):
    """Options to configure :term:`drilldown` functionality in the chart, which
    enables users to inspect increasingly high resolution data by clicking on chart
    items like columns or pie slices.

    .. note::

      The drilldown feature requires the ``drilldown.js`` file to be loaded in the
      browser/client. This file is found in the modules directory of the download
      package, or online at
      `code.highcharts.com/modules/drilldown.js <code.highcharts.com/modules/drilldown.js>`_.

    """

    def __init__(self, **kwargs):
        self._active_axis_label_style = None
        self._active_data_label_style = None
        self._allow_point_drilldown = None
        self._animation = None
        self._breadcrumbs = None
        self._drillup_button = None
        self._series = None

        self.active_axis_label_style = kwargs.get('active_axis_label_style', None)
        self.active_data_label_style = kwargs.get('active_data_label_style', None)
        self.allow_point_drilldown = kwargs.get('allow_point_drilldown', None)
        self.animation = kwargs.get('animation', None)
        self.breadcrumbs = kwargs.get('breadcrumbs', None)
        self.drillup_button = kwargs.get('drillup_button', None)
        self.series = kwargs.get('series', None)

    @property
    def _dot_path(self) -> Optional[str]:
        """The dot-notation path to the options key for the current class.
        
        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return 'drilldown'

    @property
    def active_axis_label_style(self) -> Optional[dict]:
        """Additional styles to apply to the X axis label for a point that has drilldown
        data. By default, it is underlined and blue to invite to interaction.

        Defaults to:

        .. code-block:: python

          {
              "cursor": "pointer",
              "color": "#003399",
              "fontWeight": "bold",
              "textDecoration": "underline"
          }

        In styled mode, active label styles can be set with the
        ``.highcharts-drilldown-axis-label`` class.

        :rtype: :class:`dict <python:dict>`
        """
        return self._active_axis_label_style

    @active_axis_label_style.setter
    def active_axis_label_style(self, value):
        self._active_axis_label_style = validators.dict(value, allow_empty = True)

    @property
    def active_data_label_style(self) -> Optional[dict]:
        """Additional styles to apply to the data label of a point that has drilldown
        data. By default, it is underlined and blue to invite to interaction.

        Defaults to:

        .. code-block:: python

          {
              'color': '#003399',
              'cursor': 'pointer',
              'fontWeight': 'bold',
              'textDecoration': 'underline'
          }

        In styled mode, active label styles can be set with the
        ``.highcharts-drilldown-data-label`` class.

        :rtype: :class:`dict <python:dict>`
        """
        return self._active_data_label_style

    @active_data_label_style.setter
    def active_data_label_style(self, value):
        self._active_data_label_style = validators.dict(value, allow_empty = True)

    @property
    def allow_point_drilldown(self) -> Optional[bool]:
        """If ``False``, clicking a single point will drill down all points in the same
        category, equivalent to clicking the X axis label. If ``True``, clicking a single
        point will drill down on that specific point.

        Defaults to ``True``.

        :returns: Flag which determines whether to enable or disable point drilldown.
        :rtype: :class:`bool <python:bool>` or :obj:`None <python:None>`
        """
        return self._allow_point_drilldown

    @allow_point_drilldown.setter
    def allow_point_drilldown(self, value):
        if value is None:
            self._allow_point_drilldown = None
        else:
            self._allow_point_drilldown = bool(value)

    @property
    def animation(self) -> Optional[bool | AnimationOptions]:
        """Configures the animation for all drilldown. Animation of a drilldown occurs
        when drilling between a column point and a column series, or a pie slice and a
        full pie series. Drilldown can still be used between series and points of
        different types, but animation will not occur.

        The animation can be configured as either a boolean or a :class:`AnimationOptions`
        object. If ``True``, it will apply the ``'swing'`` jQuery easing and a duration of
        500 ms by default. If used as a :class:`AnimationOptions` instance, you have more
        fine-grianed configuration control.

        :returns: Settings for the animation of image patterns.
        :rtype: :class:`AnimationOptions` or :obj:`None <python:None>`
        """
        return self._animation

    @animation.setter
    def animation(self, value):
        if value is None:
            self._animation = None
        elif value is False:
            self._animation = False
        elif value is True:
            self._animation = True
        else:
            self._animation = validate_types(value,
                                             types = AnimationOptions)

    @property
    def breadcrumbs(self) -> Optional[BreadcrumbOptions]:
        """Configuration for the breadcrumbs, the navigation at the top leading the way
        up through the drilldown levels.

        :rtype: :class:`BreadcrumbOptions` or :obj:`None <python:None>`
        """
        return self._breadcrumbs

    @breadcrumbs.setter
    @class_sensitive(BreadcrumbOptions)
    def breadcrumbs(self, value):
        self._breadcrumbs = value

    @property
    def series(self) -> Optional[List[SeriesBase]]:
        """An array of series configurations for the drilldown. These drilldown series are
        hidden by default. The drilldown series is linked to the parent series' point by
        its id.

        :rtype: :class:`list <python:list>` of :class:`SeriesBase`
        """
        return self._series

    @series.setter
    def series(self, value):
        value = validators.iterable(value, allow_empty = True)
        if not value:
            self._series = None
        else:
            self._series = [create_series_obj(x,
                                              default_type = None)
                            for x in value]

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'active_axis_label_style': as_dict.get('activeAxisLabelStyle', None),
            'active_data_label_style': as_dict.get('activeDataLabelStyle', None),
            'allow_point_drilldown': as_dict.get('allowPointDrilldown', None),
            'animation': as_dict.get('animation', None),
            'breadcrumbs': as_dict.get('breadcrumbs', None),
            'series': as_dict.get('series', None)
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'activeAxisLabelStyle': self.active_axis_label_style,
            'activeDataLabelStyle': self.active_data_label_style,
            'allowPointDrilldown': self.allow_point_drilldown,
            'animation': self.animation,
            'breadcrumbs': self.breadcrumbs,
            'series': self.series
        }

        return untrimmed



from typing import Optional, List
from decimal import Decimal

from validator_collection import validators, checkers

from highcharts_core import errors
from highcharts_core.decorators import class_sensitive, validate_types
from highcharts_core.options.plot_options.bar import BarOptions
from highcharts_core.options.plot_options.levels import LevelOptions
from highcharts_core.utility_classes.data_labels import OrganizationDataLabel


class OrganizationOptions(BarOptions):
    """General options to apply to all Organization series types.

    An organization chart is a diagram that shows the structure of an organization and
    the relationships and relative ranks of its parts and positions.

    .. tabs::

      .. tab:: Standard Organization Chart

        .. figure:: ../../../_static/organization-example.png
          :alt: Organization Example Chart
          :align: center

      .. tab:: Horizontal Organization Chart

        .. figure:: ../../../_static/organization-example-horizontal.png
          :alt: Horizontal Organization Example Chart
          :align: center

    """

    def __init__(self, **kwargs):
        self._hanging_indent = None
        self._hanging_indent_translation = None
        self._levels = None
        self._link_color = None
        self._link_line_width = None
        self._link_opacity = None
        self._link_radius = None
        self._min_link_width = None
        self._min_node_length = None
        self._node_padding = None
        self._node_width = None

        self.hanging_indent = kwargs.get('hanging_indent', None)
        self.hanging_indent_translation = kwargs.get('hanging_indent_translation', None)
        self.levels = kwargs.get('levels', None)
        self.link_color = kwargs.get('link_color', None)
        self.link_line_width = kwargs.get('link_line_width', None)
        self.link_opacity = kwargs.get('link_opacity', None)
        self.link_radius = kwargs.get('link_radius', None)
        self.min_link_width = kwargs.get('min_link_width', None)
        self.min_node_length = kwargs.get('min_node_length', None)
        self.node_padding = kwargs.get('node_padding', None)
        self.node_width = kwargs.get('node_width', None)

        super().__init__(**kwargs)

    @property
    def data_labels(self) -> Optional[OrganizationDataLabel | List[OrganizationDataLabel]]:
        """Options for the series data labels, appearing next to each data point.

        .. note::

          To have multiple data labels per data point, you can also supply a collection of
          :class:`DataLabel` configuration settings.

        :rtype: :class:`OrganizationDataLabel <highcharts_core.utility_classes.data_labels.OrganizationDataLabel>`, 
          :class:`list <python:list>` of 
            :class:`OrganizationDataLabel <highcharts_core.utility_classes.data_labels.OrganizationDataLabel>` or
            :obj:`None <python:None>`
        """
        return self._data_labels

    @data_labels.setter
    def data_labels(self, value):
        if not value:
            self._data_labels = None
        else:
            if checkers.is_iterable(value):
                self._data_labels = validate_types(value,
                                                   types = OrganizationDataLabel,
                                                   allow_none = False,
                                                   force_iterable = True)
            else:
                self._data_labels = validate_types(value,
                                                   types = OrganizationDataLabel,
                                                   allow_none = False)

    @property
    def hanging_indent(self) -> Optional[int | float | Decimal]:
        """The indentation in pixels of hanging nodes (nodes whose parent has
        ``layout`` set to ``'hanging'``). Defaults to ``20``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._hanging_indent

    @hanging_indent.setter
    def hanging_indent(self, value):
        self._hanging_indent = validators.numeric(value, allow_empty = True)

    @property
    def hanging_indent_translation(self) -> Optional[str]:
        """Defines the indentation of a ``'hanging'`` layout parent's children. Defaults
        to ``'inherit'``.

        Possible options are:

          * ``'inherit'`` - Only the first child adds the indentation, children of a child
            with indentation inherit the indentation
          * ``'cumulative'`` - All children of a child with indentation add its own
            indent. If this option causes overlapping of nodes, then use the ``'shrink'``
            setting.
          * ``'shrink'`` - Nodes shrink by the
            :meth:`hanging_indent <OrganizationOptions.hanging_indent>` value until they
            reach the :meth:`min_node_length <OrganizationOptions.min_node_length>`.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._hanging_indent_translation

    @hanging_indent_translation.setter
    def hanging_indent_translation(self, value):
        if not value:
            self._hanging_indent_translation = None
        else:
            value = validators.string(value, allow_empty = True)
            value = value.lower()
            if value not in ['inherit', 'cumulative', 'shrink']:
                raise errors.HighchartsValueError(f'hanging_indent_translation expects '
                                                  f'a value of "inherit", "cumulative", '
                                                  f'"shrink", or None. Received: {value}')

            self._hanging_indent_translation = value

    @property
    def levels(self) -> Optional[List[LevelOptions]]:
        """Set options on specific levels. Takes precedence over series options, but not
        node and link options.

        :rtype: :obj:`None <python:None>`, or :class:`list <python:list>` of
          :class:`LevelOptions`
        """
        return self._levels

    @levels.setter
    @class_sensitive(LevelOptions, force_iterable = True)
    def levels(self, value):
        self._levels = value

    @property
    def link_color(self) -> Optional[str]:
        """The color of the links between nodes. Defaults to ``'#666666'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._link_color

    @link_color.setter
    def link_color(self, value):
        self._link_color = validators.string(value, allow_empty = True)

    @property
    def link_line_width(self) -> Optional[int | float | Decimal]:
        """The line width of the links connecting nodes, in pixels. Defaults to ``1``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._link_line_width

    @link_line_width.setter
    def link_line_width(self, value):
        self._link_line_width = validators.numeric(value, allow_empty = True)

    @property
    def link_opacity(self) -> Optional[int | float | Decimal]:
        """Opacity for the links between nodes in sankey or similar diagrams. Defaults to
        ``0.5``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._link_opacity

    @link_opacity.setter
    def link_opacity(self, value):
        self._link_opacity = validators.numeric(value,
                                                allow_empty = True,
                                                minimum = 0)

    @property
    def link_radius(self) -> Optional[int | float | Decimal]:
        """Radius for the rounded corners of the links between nodes. Defaults to ``10``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._link_radius

    @link_radius.setter
    def link_radius(self, value):
        self._link_radius = validators.numeric(value, allow_empty = True)

    @property
    def min_link_width(self) -> Optional[int | float | Decimal]:
        """The minimal width for a line of a sankey. By default, 0 values are not shown.
        Defaults to ``0``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._min_link_width

    @min_link_width.setter
    def min_link_width(self, value):
        self._min_link_width = validators.numeric(value, allow_empty = True)

    @property
    def min_node_length(self) -> Optional[int | float | Decimal]:
        """In a horizontal chart, the minimum width of the hanging nodes only, expressed
        in pixels. In a vertical chart, the minimum height of the hanging nodes only,
        expressed in pixels. Defaults to ``10``.

        .. warning::

          Only applied when
          :meth:`hanging_indent_translation <OrganizationOptions.hanging_indent_translation>`
          is set to ``'shrink'``.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._min_node_length

    @min_node_length.setter
    def min_node_length(self, value):
        self._min_node_length = validators.numeric(value, allow_empty = True)

    @property
    def node_padding(self) -> Optional[int | float | Decimal]:
        """The padding between nodes in a sankey diagram or dependency wheel, in pixels.
        Defaults to ``10``.

        If the number of nodes is so great that it is possible to lay them out within the
        plot area with the given ``node_padding``, they will be rendered with a smaller
        padding as a strategy to avoid overflow.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._node_padding

    @node_padding.setter
    def node_padding(self, value):
        self._node_padding = validators.numeric(value, allow_empty = True)

    @property
    def node_width(self) -> Optional[int | float | Decimal]:
        """In a horizontal chart, the width of the nodes in pixels. Defaults to ``50``.

        .. note::

          Most organization charts are vertical, so the name of this option is
          counterintuitive.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._node_width

    @node_width.setter
    def node_width(self, value):
        self._node_width = validators.numeric(value, allow_empty = True)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'accessibility': as_dict.get('accessibility', None),
            'allow_point_select': as_dict.get('allowPointSelect', None),
            'animation': as_dict.get('animation', None),
            'class_name': as_dict.get('className', None),
            'clip': as_dict.get('clip', None),
            'color': as_dict.get('color', None),
            'cursor': as_dict.get('cursor', None),
            'custom': as_dict.get('custom', None),
            'dash_style': as_dict.get('dashStyle', None),
            'data_labels': as_dict.get('dataLabels', None),
            'description': as_dict.get('description', None),
            'enable_mouse_tracking': as_dict.get('enableMouseTracking', None),
            'events': as_dict.get('events', None),
            'include_in_data_export': as_dict.get('includeInDataExport', None),
            'keys': as_dict.get('keys', None),
            'label': as_dict.get('label', None),
            'legend_symbol': as_dict.get('legendSymbol', None),
            'linked_to': as_dict.get('linkedTo', None),
            'marker': as_dict.get('marker', None),
            'on_point': as_dict.get('onPoint', None),
            'opacity': as_dict.get('opacity', None),
            'point': as_dict.get('point', None),
            'point_description_formatter': as_dict.get('pointDescriptionFormatter', None),
            'selected': as_dict.get('selected', None),
            'show_checkbox': as_dict.get('showCheckbox', None),
            'show_in_legend': as_dict.get('showInLegend', None),
            'skip_keyboard_navigation': as_dict.get('skipKeyboardNavigation', None),
            'sonification': as_dict.get('sonification', None),
            'states': as_dict.get('states', None),
            'sticky_tracking': as_dict.get('stickyTracking', None),
            'threshold': as_dict.get('threshold', None),
            'tooltip': as_dict.get('tooltip', None),
            'turbo_threshold': as_dict.get('turboThreshold', None),
            'visible': as_dict.get('visible', None),

            'animation_limit': as_dict.get('animationLimit', None),
            'boost_blending': as_dict.get('boostBlending', None),
            'boost_threshold': as_dict.get('boostThreshold', None),
            'color_axis': as_dict.get('colorAxis', None),
            'color_index': as_dict.get('colorIndex', None),
            'color_key': as_dict.get('colorKey', None),
            'connect_ends': as_dict.get('connectEnds', None),
            'connect_nulls': as_dict.get('connectNulls', None),
            'crisp': as_dict.get('crisp', None),
            'crop_threshold': as_dict.get('cropThreshold', None),
            'data_sorting': as_dict.get('dataSorting', None),
            'drag_drop': as_dict.get('dragDrop', None),
            'fill_color': as_dict.get('fillColor', None),
            'fill_opacity': as_dict.get('fillOpacity', None),
            'find_nearest_point_by': as_dict.get('findNearestPointBy', None),
            'get_extremes_from_all': as_dict.get('getExtremesFromAll', None),
            'linecap': as_dict.get('linecap', None),
            'line_color': as_dict.get('lineColor', None),
            'line_width': as_dict.get('lineWidth', None),
            'negative_color': as_dict.get('negativeColor', None),
            'negative_fill_color': as_dict.get('negativeFillColor', None),
            'point_interval': as_dict.get('pointInterval', None),
            'point_interval_unit': as_dict.get('pointIntervalUnit', None),
            'point_placement': as_dict.get('pointPlacement', None),
            'point_start': as_dict.get('pointStart', None),
            'relative_x_value': as_dict.get('relativeXValue', None),
            'shadow': as_dict.get('shadow', None),
            'soft_threshold': as_dict.get('softThreshold', None),
            'stacking': as_dict.get('stacking', None),
            'step': as_dict.get('step', None),
            'track_by_area': as_dict.get('trackByArea', None),
            'zone_axis': as_dict.get('zoneAxis', None),
            'zones': as_dict.get('zones', None),

            'border_color': as_dict.get('borderColor', None),
            'border_radius': as_dict.get('borderRadius', None),
            'border_width': as_dict.get('borderWidth', None),
            'center_in_category': as_dict.get('centerInCategory', None),
            'color_by_point': as_dict.get('colorByPoint', None),
            'colors': as_dict.get('colors', None),
            'grouping': as_dict.get('grouping', None),
            'group_padding': as_dict.get('groupPadding', None),
            'max_point_width': as_dict.get('maxPointWidth', None),
            'min_point_length': as_dict.get('minPointLength', None),
            'point_padding': as_dict.get('pointPadding', None),
            'point_range': as_dict.get('pointRange', None),
            'point_width': as_dict.get('pointWidth', None),

            'depth': as_dict.get('depth', None),
            'edge_color': as_dict.get('edgeColor', None),
            'edge_width': as_dict.get('edgeWidth', None),
            'group_z_padding': as_dict.get('groupZPadding', None),

            'hanging_indent': as_dict.get('hangingIndent', None),
            'hanging_indent_translation': as_dict.get('hangingIndentTranslation', None),
            'levels': as_dict.get('levels', None),
            'link_color': as_dict.get('linkColor', None),
            'link_line_width': as_dict.get('linkLineWidth', None),
            'link_opacity': as_dict.get('linkOpacity', None),
            'link_radius': as_dict.get('linkRadius', None),
            'min_link_width': as_dict.get('minLinkWidth', None),
            'min_node_length': as_dict.get('minNodeLength', None),
            'node_padding': as_dict.get('nodePadding', None),
            'node_width': as_dict.get('nodeWidth', None)
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'hangingIndent': self.hanging_indent,
            'hangingIndentTranslation': self.hanging_indent_translation,
            'levels': self.levels,
            'linkColor': self.link_color,
            'linkLineWidth': self.link_line_width,
            'linkOpacity': self.link_opacity,
            'linkRadius': self.link_radius,
            'minLinkWidth': self.min_link_width,
            'minNodeLength': self.min_node_length,
            'nodePadding': self.node_padding,
            'nodeWidth': self.node_width
        }
        parent_as_dict = super()._to_untrimmed_dict(in_cls = in_cls)

        for key in parent_as_dict:
            untrimmed[key] = parent_as_dict[key]

        return untrimmed



from typing import Optional

from validator_collection import validators

from highcharts_core import constants
from highcharts_core.decorators import class_sensitive
from highcharts_core.metaclasses import HighchartsMeta


class ScreenReaderSectionAnnotationLanguage(HighchartsMeta):
    """Language options for annotation descriptions."""

    def __init__(self, **kwargs):
        self._description_multiple_points = None
        self._description_no_points = None
        self._description_single_point = None
        self._heading = None

        self.description_multiple_points = kwargs.get('description_multiple_points', None)
        self.description_no_points = kwargs.get('description_no_points', None)
        self.description_single_point = kwargs.get('description_single_point', None)
        self.heading = kwargs.get('heading', None)

    @property
    def description_multiple_points(self) -> Optional[str]:
        """Defaults to
        ``'{annotationText}. Related to {annotationPoint}{ Also related to, #each(additionalAnnotationPoints)}'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._description_multiple_points

    @description_multiple_points.setter
    def description_multiple_points(self, value):
        self._description_multiple_points = validators.string(value, allow_empty = True)

    @property
    def description_no_points(self) -> Optional[str]:
        """Defaults to
        ``'{annotationText}'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._description_no_points

    @description_no_points.setter
    def description_no_points(self, value):
        self._description_no_points = validators.string(value,
                                                        allow_empty = True)

    @property
    def description_single_point(self) -> Optional[str]:
        """Defaults to
        ``'{annotationText}. Related to {annotationPoint}'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._description_single_point

    @description_single_point.setter
    def description_single_point(self, value):
        self._description_single_point = validators.string(value, allow_empty = True)

    @property
    def heading(self) -> Optional[str]:
        """Defaults to
        ``'Chart annotations summary'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._heading

    @heading.setter
    def heading(self, value):
        self._heading = validators.string(value, allow_empty = True)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'description_multiple_points': as_dict.get('descriptionMultiplePoints', None),
            'description_no_points': as_dict.get('descriptionNoPoints', None),
            'description_single_point': as_dict.get('descriptionSinglePoint', None),
            'heading': as_dict.get('heading', None),
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'descriptionMultiplePoints': self.description_multiple_points,
            'descriptionNoPoints': self.description_no_points,
            'descriptionSinglePoint': self.description_single_point,
            'heading': self.heading
        }

        return untrimmed


class ScreenReaderSectionLanguageOptions(HighchartsMeta):
    """Language options for the screen reader information sections added before and
    after the chart when used in accessibility mode."""

    def __init__(self, **kwargs):
        self._after_region_label = None
        self._annotations = None
        self._before_region_label = None
        self._end_of_chart_marker = None

        self.after_region_label = kwargs.get('after_region_label', None)
        self.annotations = kwargs.get('annotations', None)
        self.before_region_label = kwargs.get('before_region_label', None)
        self.end_of_chart_marker = kwargs.get('end_of_chart_marker', None)

    @property
    def after_region_label(self) -> Optional[str]:
        """Defaults to ``''`` (empty
        string).

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._after_region_label

    @after_region_label.setter
    def after_region_label(self, value):
        self._after_region_label = validators.string(value, allow_empty = True)

    @property
    def annotations(self) -> Optional[ScreenReaderSectionAnnotationLanguage]:
        """Language options for annotation descriptions.

        :rtype: :class:`ScreenReaderSectionAnnotationLanguage` or
          :obj:`None <python:None>`
        """
        return self._annotations

    @annotations.setter
    @class_sensitive(ScreenReaderSectionAnnotationLanguage)
    def annotations(self, value):
        self._annotations = value

    @property
    def before_region_label(self) -> Optional[str]:
        """Defaults to ``''`` (empty
        string).

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._before_region_label

    @before_region_label.setter
    def before_region_label(self, value):
        self._before_region_label = validators.string(value, allow_empty = True)

    @property
    def end_of_chart_marker(self) -> Optional[str]:
        """Defaults to ``'End of interactive chart.'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._end_of_chart_marker

    @end_of_chart_marker.setter
    def end_of_chart_marker(self, value):
        self._end_of_chart_marker = validators.string(value, allow_empty = True)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'after_region_label': as_dict.get('afterRegionLabel', None),
            'annotations': as_dict.get('annotations', None),
            'before_region_label': as_dict.get('beforeRegionLabel', None),
            'end_of_chart_marker': as_dict.get('endOfChartMarker', None),
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'afterRegionLabel': self.after_region_label,
            'annotations': self.annotations,
            'beforeRegionLabel': self.before_region_label,
            'endOfChartMarker': self.end_of_chart_marker
        }

        return untrimmed



"""Tests for ``highcharts.no_data``."""

import pytest

from json.decoder import JSONDecodeError

from highcharts_core.options.legend.bubble_legend import BubbleLegend as cls
from highcharts_core import errors
from tests.fixtures import input_files, check_input_file, to_camelCase, to_js_dict, \
    Class__init__, Class__to_untrimmed_dict, Class_from_dict, Class_to_dict, \
    Class_from_js_literal

STANDARD_PARAMS = [
    ({}, None),
    ({
      'border_color': '#ccc',
      'border_width': 12,
      'class_name': 'some-class-name',
      'color': '#fff',
      'connector_class_name': 'some-class-name',
      'connector_color': '#999',
      'connector_distance': 6,
      'connector_width': 2,
      'enabled': True,
      'labels': {
          'align': 'center',
          'allowOverlap': False,
          'className': 'some-class-name',
          'format': 'a format string',
          'formatter': """function () { return true; }""",
          'style': 'some style string',
          'x': 3,
          'y': 1
      },
      'legend_index': 1,
      'max_size': 24,
      'min_size': 6,
      'ranges': [
          {
           'borderColor': '#ccc',
           'color': '#fff',
           'connectorColor': '#999',
           'value': 123
          },
          {
           'borderColor': '#ccc',
           'color': '#fff',
           'connectorColor': '#999',
           'value': 123
          },
          {
           'borderColor': '#ccc',
           'color': '#fff',
           'connectorColor': '#999',
           'value': 123
          }
      ],
      'size_by': 'width',
      'size_by_absolute_value': True,
      'z_index': 3,
      'z_threshold': 123
    }, None),
]


@pytest.mark.parametrize('kwargs, error', STANDARD_PARAMS)
def test__init__(kwargs, error):
    Class__init__(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error', STANDARD_PARAMS)
def test__to_untrimmed_dict(kwargs, error):
    Class__to_untrimmed_dict(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error',  STANDARD_PARAMS)
def test_from_dict(kwargs, error):
    Class_from_dict(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error',  STANDARD_PARAMS)
def test_to_dict(kwargs, error):
    Class_to_dict(cls, kwargs, error)


@pytest.mark.parametrize('filename, as_file, error', [
    ('legend/bubble_legend/01.js', False, None),

    ('legend/bubble_legend/error-01.js', False, (errors.HighchartsValueError,
                                                 errors.HighchartsParseError,
                                                 JSONDecodeError,
                                                 TypeError,
                                                 ValueError)),

    ('legend/bubble_legend/01.js', True, None),

    ('legend/bubble_legend/error-01.js', True, (errors.HighchartsValueError,
                                                errors.HighchartsParseError,
                                                JSONDecodeError,
                                                TypeError,
                                                ValueError)),
])
def test_from_js_literal(input_files, filename, as_file, error):
    Class_from_js_literal(cls, input_files, filename, as_file, error)



"""Tests for ``highcharts.no_data``."""

import pytest

from json.decoder import JSONDecodeError

from highcharts_core.chart import Chart as cls
from highcharts_core.global_options.shared_options import SharedOptions
from highcharts_core import errors
from tests.fixtures import input_files, check_input_file, to_camelCase, to_js_dict, \
    Class__init__, Class__to_untrimmed_dict, Class_from_dict, Class_to_dict, \
    Class_from_js_literal, Class_from_js_literal_with_expected

STANDARD_PARAMS = [
    ({}, None),
]


@pytest.mark.parametrize('kwargs, error', STANDARD_PARAMS)
def test__init__(kwargs, error):
    Class__init__(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error', [
    ({}, errors.HighchartsValueError),
])
def test_bugfix34_SharedOptions_error(kwargs, error):
    shared_options = SharedOptions()
    instance = cls(**kwargs)
    if not error:
        instance.options = shared_options
    else:
        with pytest.raises(error):
            instance.options = shared_options


@pytest.mark.parametrize('kwargs, error', STANDARD_PARAMS)
def test__to_untrimmed_dict(kwargs, error):
    Class__to_untrimmed_dict(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error',  STANDARD_PARAMS)
def test_from_dict(kwargs, error):
    Class_from_dict(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error',  STANDARD_PARAMS)
def test_to_dict(kwargs, error):
    Class_to_dict(cls, kwargs, error)


@pytest.mark.parametrize('input_filename, expected_filename, as_file, error', [
    ('chart_obj/01-input.js',
     'chart_obj/01-expected.js',
     False,
     None),
    ('chart_obj/01-input.js',
     'chart_obj/01-expected.js',
     False,
     None),
])
def test_from_js_literal(input_files, input_filename, expected_filename, as_file, error):
    Class_from_js_literal_with_expected(cls,
                                        input_files,
                                        input_filename,
                                        expected_filename,
                                        as_file,
                                        error)



@pytest.mark.parametrize('json_str, expected_modules, error', [
    ("""{
    "chart": {
        "type": "column"
    },
    "colors": null,
    "credits": false,
    "exporting": {
        "scale": 1
    },
    "series": [{
        "baseSeries": 1,
        "color": "#434343",
        "name": "Pareto",
        "tooltip": {
            "valueDecimals": 2,
            "valueSuffix": "%"
        },
        "type": "pareto",
        "yAxis": 1,
        "zIndex": 10
    }, {
        "color": "#7cb5ec",
        "data": [1, 23, 45, 54, 84, 13, 8, 7, 23, 1, 34, 6, 8, 99, 85, 23, 3, 3, 3, 3, 3, 2, 2, 2, 2, 1, 1, 1, 1],
        "name": "random-name",
        "type": "column",
        "zIndex": 2
    }],
    "title": {
        "text": "Random Name Pareto"
    },
    "tooltip": {
        "shared": true
    },
    "xAxis": {
        "categories": ["Something", "Something", "Something", "Something", "Something", "Something", "Hypovolemia", "Something", "Something", "Something", "Something", "Something", "Something", "Something", "Something", "Something", "Something", "Something", "Something", "Something", "Something", "Something", "Something", "Something", "Something", "Something", "Something", "Something", "Something"],
        "crosshair": true,
        "labels": {
            "rotation": 90
        }
    },
    "yAxis": [{
        "title": {
            "text": "count"
        }
    }, {
        "labels": {
            "format": "{value}%"
        },
        "max": 100,
        "maxPadding": 0,
        "min": 0,
        "minPadding": 0,
        "opposite": true,
        "title": {
            "text": "accum percent"
        }
    }]
}""", ['highcharts', 'modules/exporting', 'modules/pareto'], None),
])
def test_get_required_modules(json_str, expected_modules, error):
    from highcharts_core.options import HighchartsOptions
    options = HighchartsOptions.from_json(json_str)
    chart = cls.from_options(options)
    if not error:
        result = chart.get_required_modules()
        if expected_modules:
            assert len(result) == len(expected_modules)
            for item in expected_modules:
                assert item in result
        else:
            assert result is None or len(result) == 0
    else:
        with pytest.raises(error):
            result = chart.get_required_modules()



"""Tests for ``highcharts.no_data``."""

import pytest

from json.decoder import JSONDecodeError

from highcharts_core.options.legend import Legend as cls
from highcharts_core import errors
from tests.fixtures import input_files, check_input_file, to_camelCase, to_js_dict, \
    Class__init__, Class__to_untrimmed_dict, Class_from_dict, Class_to_dict, \
    Class_from_js_literal

STANDARD_PARAMS = [
    ({}, None),
    ({
      'accessibility': {
          'enabled': True,
          'keyboardNavigation': {
              'enabled': True
          }
      },
      'align': 'right',
      'align_columns': True,
      'background_color': '#fff',
      'border_color': '#ccc',
      'border_radius': 4,
      'class_name': 'some-class-name',
      'enabled': True,
      'floating': False,
      'item_checkbox_style': 'some-style-setting',
      'item_distance': 12,
      'item_hidden_style': 'some-style-setting',
      'item_hover_style': 'some-style-setting',
      'item_margin_bottom': 7,
      'item_margin_top': 7,
      'item_style': 'some-style-setting',
      'item_width': 36,
      'label_format': 'format string',
      'label_formatter': """function () { return true; }""",
      'layout': 'vertical',
      'margin': 7,
      'max_height': 120,
      'padding': 8,
      'reversed': False,
      'rtl': False,
      'shadow': False,
      'square_symbol': False,
      'symbol_height': 12,
      'symbol_padding': 2,
      'symbol_radius': 0,
      'symbol_width': 12,
      'title': {
          'style': 'some-style-string',
          'text': 'Legend Title Goes Here'
      },
      'use_html': False,
      'vertical_align': 'top',
      'width': 120,
      'x': 0,
      'y': 0
    }, None),
]


@pytest.mark.parametrize('kwargs, error', STANDARD_PARAMS)
def test__init__(kwargs, error):
    Class__init__(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error', STANDARD_PARAMS)
def test__to_untrimmed_dict(kwargs, error):
    Class__to_untrimmed_dict(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error',  STANDARD_PARAMS)
def test_from_dict(kwargs, error):
    Class_from_dict(cls, kwargs, error)


@pytest.mark.parametrize('kwargs, error',  STANDARD_PARAMS)
def test_to_dict(kwargs, error):
    Class_to_dict(cls, kwargs, error)


@pytest.mark.parametrize('filename, as_file, error', [
    ('legend/legend/01.js', False, None),

    ('legend/legend/error-01.js', False, (errors.HighchartsValueError,
                                          errors.HighchartsParseError,
                                          JSONDecodeError,
                                          TypeError,
                                          ValueError)),

    ('legend/legend/01.js', True, None),

    ('legend/legend/error-01.js', True, (errors.HighchartsValueError,
                                         errors.HighchartsParseError,
                                         JSONDecodeError,
                                         TypeError,
                                         ValueError)),
])
def test_from_js_literal(input_files, filename, as_file, error):
    Class_from_js_literal(cls, input_files, filename, as_file, error)



from typing import Optional
from decimal import Decimal

from validator_collection import validators, checkers

from highcharts_core import constants, errors
from highcharts_core.decorators import class_sensitive
from highcharts_core.options.series.data.single_point import SinglePointData
from highcharts_core.options.plot_options.drag_drop import DragDropOptions
from highcharts_core.utility_classes.data_labels import DataLabel


class PieData(SinglePointData):
    """Data point that can be represented as a slice on a :class:`PieSeries` with a single
    ``y`` value."""

    def __init__(self, **kwargs):
        self._sliced = None

        self.sliced = kwargs.get('sliced', None)

        super().__init__(**kwargs)

    @property
    def sliced(self) -> Optional[bool]:
        """If ``True``, displays the slice offset from the center. Defaults to
        :obj:`None <python:None>`

        :rtype: :class:`bool <python:bool>` or :obj:`None <python:None>`
        """
        return self._sliced

    @sliced.setter
    def sliced(self, value):
        if value is None:
            self._sliced = None
        else:
            self._sliced = bool(value)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        """Convenience method which returns the keyword arguments used to initialize the
        class from a Highcharts Javascript-compatible :class:`dict <python:dict>` object.

        :param as_dict: The HighCharts JS compatible :class:`dict <python:dict>`
          representation of the object.
        :type as_dict: :class:`dict <python:dict>`

        :returns: The keyword arguments that would be used to initialize an instance.
        :rtype: :class:`dict <python:dict>`

        """
        kwargs = {
            'accessibility': as_dict.get('accessibility', None),
            'class_name': as_dict.get('className', None),
            'color': as_dict.get('color', None),
            'color_index': as_dict.get('colorIndex', None),
            'custom': as_dict.get('custom', None),
            'description': as_dict.get('description', None),
            'events': as_dict.get('events', None),
            'id': as_dict.get('id', None),
            'label_rank': as_dict.get('labelrank', None),
            'name': as_dict.get('name', None),
            'selected': as_dict.get('selected', None),

            'data_labels': as_dict.get('dataLabels', None),
            'drag_drop': as_dict.get('dragDrop', None),
            'drilldown': as_dict.get('drilldown', None),
            'sliced': as_dict.get('sliced', None),
            'y': as_dict.get('y', None),
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'sliced': self.sliced,
        }

        parent_as_dict = super()._to_untrimmed_dict(in_cls = in_cls) or {}
        for key in parent_as_dict:
            untrimmed[key] = parent_as_dict[key]

        return untrimmed


class VariablePieData(PieData):
    """Variant of :class:`PieData` suited for :class:`VariablePieSeries`."""

    def __init__(self, **kwargs):
        self._sliced = None
        self._z = None

        self.sliced = kwargs.get('sliced', None)
        self.z = kwargs.get('z', None)

        super().__init__(**kwargs)

    @property
    def z(self) -> Optional[int | float | Decimal | type(None) | constants.EnforcedNullType]:
        """The position of the data point on the Z-axis. Defaults to
        :obj:`None <python:None>`.

        :rtype: numeric or :class:`EnforcedNullType` or :obj:`None <python:None>`
        """
        return self._z

    @z.setter
    def z(self, value):
        if value is None or isinstance(value, constants.EnforcedNullType):
            self._z = None
        else:
            self._z = validators.numeric(value)

    @classmethod
    def from_array(cls, value):
        if not value:
            return []
        elif checkers.is_string(value):
            try:
                value = validators.json(value)
            except (ValueError, TypeError):
                pass
        elif not checkers.is_iterable(value):
            value = [value]

        collection = []
        for item in value:
            if checkers.is_type(item, 'VariablePieData'):
                as_obj = item
            elif checkers.is_dict(item):
                as_obj = cls.from_dict(item)
            elif item is None or isinstance(item, constants.EnforcedNullType):
                as_obj = cls(y = None,
                             z = None)
            elif len(item) == 2:
                as_obj = cls(y = item[0],
                             z = item[1])
            else:
                raise errors.HighchartsValueError(f'each data point supplied must either '
                                                  f'be a Variable Pie Data Point or be '
                                                  f'coercable to one. Could not coerce: '
                                                  f'{item}')
            collection.append(as_obj)

        return collection

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        """Convenience method which returns the keyword arguments used to initialize the
        class from a Highcharts Javascript-compatible :class:`dict <python:dict>` object.

        :param as_dict: The HighCharts JS compatible :class:`dict <python:dict>`
          representation of the object.
        :type as_dict: :class:`dict <python:dict>`

        :returns: The keyword arguments that would be used to initialize an instance.
        :rtype: :class:`dict <python:dict>`

        """
        kwargs = {
            'accessibility': as_dict.get('accessibility', None),
            'class_name': as_dict.get('className', None),
            'color': as_dict.get('color', None),
            'color_index': as_dict.get('colorIndex', None),
            'custom': as_dict.get('custom', None),
            'description': as_dict.get('description', None),
            'events': as_dict.get('events', None),
            'id': as_dict.get('id', None),
            'label_rank': as_dict.get('labelrank', None),
            'name': as_dict.get('name', None),
            'selected': as_dict.get('selected', None),

            'data_labels': as_dict.get('dataLabels', None),
            'drag_drop': as_dict.get('dragDrop', None),
            'drilldown': as_dict.get('drilldown', None),
            'sliced': as_dict.get('sliced', None),
            'y': as_dict.get('y', None),

            'z': as_dict.get('z', None),
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'z': self.z,
        }

        parent_as_dict = super()._to_untrimmed_dict(in_cls = in_cls) or {}
        for key in parent_as_dict:
            untrimmed[key] = parent_as_dict[key]

        return untrimmed



from typing import Optional
from decimal import Decimal

from validator_collection import validators

from highcharts_core.decorators import class_sensitive
from highcharts_core.options.plot_options.generic import GenericTypeOptions
from highcharts_core.utility_classes.clusters import ClusterOptions


class VennOptions(GenericTypeOptions):
    """General options to apply to all Venn series types.

    A Venn diagram displays all possible logical relations between a collection of
    different sets. The sets are represented by circles, and the relation between the
    sets are displayed by the overlap or lack of overlap between them. The venn
    diagram is a special case of Euler diagrams, which can also be displayed by this
    series type.

    .. tabs::

      .. tab:: Venn Diagram

        .. figure:: ../../../_static/venn-example.png
          :alt: Venn Example Chart
          :align: center

      .. tab:: Euler Diagram

        .. figure:: ../../../_static/venn-example-euler.png
          :alt: Euler Example Chart
          :align: center

    """

    def __init__(self, **kwargs):
        self._animation_limit = None
        self._color_axis = None
        self._color_index = None
        self._color_key = None
        self._crisp = None
        self._relative_x_value = False
        self._step = None

        self._border_dash_style = None

        self._brighten = None
        self._cluster = None

        self.animation_limit = kwargs.get('animation_limit', None)
        self.color_axis = kwargs.get('color_axis', None)
        self.color_index = kwargs.get('color_index', None)
        self.color_key = kwargs.get('color_key', None)
        self.crisp = kwargs.get('crisp', None)
        self.relative_x_value = kwargs.get('relative_x_value', None)
        self.step = kwargs.get('step', None)

        self.border_dash_style = kwargs.get('border_dash_style', None)
        self.brighten = kwargs.get('brighten', None)
        self.cluster = kwargs.get('cluster', None)

        super().__init__(**kwargs)

    @property
    def animation_limit(self) -> Optional[int | float | Decimal]:
        """For some series, there is a limit that shuts down initial animation by default
        when the total number of points in the chart is too high. Defaults to
        :obj:`None <python:None>`.

        For example, for a column chart and its derivatives, animation does not run if
        there is more than 250 points totally. To disable this cap, set
        ``animation_limit`` to ``float("inf")`` (which represents infinity).

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._animation_limit

    @animation_limit.setter
    def animation_limit(self, value):
        if value == float('inf'):
            self._animation_limit = float('inf')
        else:
            self._animation_limit = validators.numeric(value,
                                                       allow_empty = True,
                                                       minimum = 0)

    @property
    def border_dash_style(self) -> Optional[str]:
        """The dash style to apply to the border. Defaults to ``'solid'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._border_dash_style

    @border_dash_style.setter
    def border_dash_style(self, value):
        self._border_dash_style = validators.string(value, allow_empty = True)

    @property
    def brighten(self) -> Optional[int | float | Decimal]:
        """Defaults to ``0``. Factor by which to brighten the diagram.

        .. warning::

          This feature is not well documented, so this is merely a "guess" as to its
          functionality.

        :rtype: numeric or :obj:`None <python:None>`
        """
        return self._brighten

    @brighten.setter
    def brighten(self, value):
        self._brighten = validators.numeric(value, allow_empty = True)

    @property
    def cluster(self) -> Optional[ClusterOptions]:
        """Options for marker clusters, the concept of sampling the data values into
        larger blocks in order to ease readability and increase performance of the
        JavaScript charts.

        .. warning::

          The marker clusters module does not work with ``boost`` and ``draggable-points``
          modules.

        .. note::

          The marker clusters feature requires the ``marker-clusters.js`` file to be
          loaded, found in the modules directory of the download package, or online at
          `code.highcharts.com/modules/marker-clusters.js <code.highcharts.com/modules/marker-clusters.js>`_.

        """
        return self._cluster

    @cluster.setter
    @class_sensitive(ClusterOptions)
    def cluster(self, value):
        self._cluster = value

    @property
    def color_axis(self) -> Optional[str | int | bool]:
        """When using dual or multiple color axes, this setting defines which
        :term:`color axis` the particular series is connected to. It refers to either the
        :meth:`ColorAxis.id` or the index of the axis in the :class:`ColorAxis` array,
        with ``0`` being the first. Set this option to ``False`` to prevent a series from
        connecting to the default color axis.

        Defaults to ``0``.

        :rtype: :obj:`None <python:None>` or :class:`str <python:str>` or
          :class:`int <python:int>` or :class:`bool <python:bool>`
        """
        return self._color_axis

    @color_axis.setter
    def color_axis(self, value):
        if value is None:
            self._color_axis = None
        elif value is False:
            self._color_axis = False
        else:
            try:
                self._color_axis = validators.string(value)
            except TypeError:
                self._color_axis = validators.integer(value,
                                                      minimum = 0)

    @property
    def color_index(self) -> Optional[int]:
        """When operating in :term:`styled mode`, a specific color index to use for the
        series, so that its graphic representations are given the class name
        ``highcharts-color-{n}``.

        Defaults to :obj:`None <python:None>`.

        :rtype: :class:`int <python:int>` or :obj:`None <python:None>`
        """
        return self._color_index

    @color_index.setter
    def color_index(self, value):
        self._color_index = validators.integer(value,
                                               allow_empty = True,
                                               minimum = 0)

    @property
    def color_key(self) -> Optional[str]:
        """Determines what data value should be used to calculate point color if
        :meth:`AreaOptions.color_axis` is used.

        .. note::

          Requires to set ``min`` and ``max`` if some custom point property is used or if
          approximation for data grouping is set to ``'sum'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._color_key

    @color_key.setter
    def color_key(self, value):
        self._color_key = validators.string(value, allow_empty = True)

    @property
    def crisp(self) -> Optional[bool]:
        """If ``True``, each point or column edge is rounded to its nearest pixel in order
        to render sharp on screen. Defaults to ``True``.

        .. hint::

          In some cases, when there are a lot of densely packed columns, this leads to
          visible difference in column widths or distance between columns. In these cases,
          setting ``crisp`` to ``False`` may look better, even though each column is
          rendered blurry.

        :rtype: :class:`bool <python:bool>` or :obj:`None <python:None>`
        """
        return self._crisp

    @crisp.setter
    def crisp(self, value):
        if value is None:
            self._crisp = None
        else:
            self._crisp = bool(value)

    @property
    def relative_x_value(self) -> Optional[bool]:
        """When ``True``, X values in the data set are relative to the current
        :meth:`point_start <AreaOptions.point_start>`,
        :meth:`point_interval <AreaOptions.point_interval>`, and
        :meth:`point_interval_unit <AreaOptions.point_interval_unit>` settings. This
        allows compression of the data for datasets with irregular X values. Defaults to
        ``False``.

        The real X values are computed on the formula ``f(x) = ax + b``, where ``a`` is
        the :meth:`point_interval <AreaOptions.point_interval>` (optionally with a time
        unit given by :meth:`point_interval_unit <AreaOptions.point_interval_unit>`), and
        ``b`` is the :meth:`point_start <AreaOptions.point_start>`.

        :rtype: :class:`bool <python:bool>` or :obj:`None <python:None>`
        """
        return self._relative_x_value

    @relative_x_value.setter
    def relative_x_value(self, value):
        if value is None:
            self._relative_x_value = None
        else:
            self._relative_x_value = bool(value)

    @property
    def step(self) -> Optional[str]:
        """Whether to apply steps to the line. Defaults to :obj:`None <python:None>`.

        Possible values are:

          * :obj:`None <python:None>`
          * ``'left'``
          * ``'center'``
          * ``'right'``

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._step

    @step.setter
    def step(self, value):
        self._step = validators.string(value, allow_empty = True)

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        """Convenience method which returns the keyword arguments used to initialize the
        class from a Highcharts Javascript-compatible :class:`dict <python:dict>` object.

        :param as_dict: The HighCharts JS compatible :class:`dict <python:dict>`
          representation of the object.
        :type as_dict: :class:`dict <python:dict>`

        :returns: The keyword arguments that would be used to initialize an instance.
        :rtype: :class:`dict <python:dict>`

        """
        kwargs = {
            'accessibility': as_dict.get('accessibility', None),
            'allow_point_select': as_dict.get('allowPointSelect', None),
            'animation': as_dict.get('animation', None),
            'class_name': as_dict.get('className', None),
            'clip': as_dict.get('clip', None),
            'color': as_dict.get('color', None),
            'cursor': as_dict.get('cursor', None),
            'custom': as_dict.get('custom', None),
            'dash_style': as_dict.get('dashStyle', None),
            'data_labels': as_dict.get('dataLabels', None),
            'description': as_dict.get('description', None),
            'enable_mouse_tracking': as_dict.get('enableMouseTracking', None),
            'events': as_dict.get('events', None),
            'include_in_data_export': as_dict.get('includeInDataExport', None),
            'keys': as_dict.get('keys', None),
            'label': as_dict.get('label', None),
            'legend_symbol': as_dict.get('legendSymbol', None),
            'linked_to': as_dict.get('linkedTo', None),
            'marker': as_dict.get('marker', None),
            'on_point': as_dict.get('onPoint', None),
            'opacity': as_dict.get('opacity', None),
            'point': as_dict.get('point', None),
            'point_description_formatter': as_dict.get('pointDescriptionFormatter', None),
            'selected': as_dict.get('selected', None),
            'show_checkbox': as_dict.get('showCheckbox', None),
            'show_in_legend': as_dict.get('showInLegend', None),
            'skip_keyboard_navigation': as_dict.get('skipKeyboardNavigation', None),
            'sonification': as_dict.get('sonification', None),
            'states': as_dict.get('states', None),
            'sticky_tracking': as_dict.get('stickyTracking', None),
            'threshold': as_dict.get('threshold', None),
            'tooltip': as_dict.get('tooltip', None),
            'turbo_threshold': as_dict.get('turboThreshold', None),
            'visible': as_dict.get('visible', None),

            'animation_limit': as_dict.get('animationLimit', None),
            'color_axis': as_dict.get('colorAxis', None),
            'color_index': as_dict.get('colorIndex', None),
            'color_key': as_dict.get('colorKey', None),
            'crisp': as_dict.get('crisp', None),
            'relative_x_value': as_dict.get('relativeXValue', None),
            'step': as_dict.get('step', None),

            'border_dash_style': as_dict.get('borderDashStyle', None),
            'brighten': as_dict.get('brighten', None),
            'cluster': as_dict.get('cluster', None)
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'animationLimit': self.animation_limit,
            'borderDashStyle': self.border_dash_style,
            'brighten': self.brighten,
            'cluster': self.cluster,
            'colorAxis': self.color_axis,
            'colorIndex': self.color_index,
            'colorKey': self.color_key,
            'crisp': self.crisp,
            'relativeXValue': self.relative_x_value,
            'step': self.step
        }

        parent_as_dict = super()._to_untrimmed_dict(in_cls = in_cls) or {}
        for key in parent_as_dict:
            untrimmed[key] = parent_as_dict[key]

        return untrimmed



"""Collection of utility functions used across the library."""
import csv
import string
import random

from validator_collection import validators

from highcharts_core import errors


def get_random_string(length = 6):
    """Generate a short random alphanumeric string.
    
    :param length: The length of the string to generate. Defaults to ``8``.
    :type length: :class:`int <python:int>`
    
    :returns: A random alphanumeric string of length ``length``.
    :rtype: :class:`str <python:str>`
    """
    length = validators.integer(length, minimum = 1)
    result = ''.join(random.choices(string.ascii_uppercase + string.digits,
                                    k = length))

    return str(result)


def mro_to_dict(obj):
    """Work through ``obj``'s multiple parent classes, executing the appropriate
    ``to_dict()`` method for each parent and consolidaitng the results to a single
    :class:`dict <python:dict>`.

    :param obj: An object that has a ``to_dict()`` method.

    :rtype: :class:`dict <python:dict>`
    """
    if not hasattr(obj, 'to_dict'):
        raise TypeError('obj does not have a to_dict() method.')

    classes = [x for x in obj.__class__.mro()
               if x.__name__ != 'object']
    as_dict = {}

    for item in classes:
        has_to_dict = hasattr(super(item, obj), 'to_dict')
        if not has_to_dict:
            break

        try:
            item_dict = super(item, obj).to_dict()
        except (NotImplementedError, AttributeError):
            continue
        for key in item_dict:
            as_dict[key] = item_dict[key]

    return as_dict


def get_remaining_mro(cls,
                      in_cls = None,
                      method = '_to_untrimmed_dict'):
    """Retrieve the remaining classes that should be processed for ``method`` when
    traversing ``cls``.

    :param cls: The class whose ancestors are being traversed.
    :type cls: :class:`HighchartsMeta`

    :param in_cls: The class that the traversal currently finds itself in. Defaults to
      :obj:`None <python:None>`
    :type in_cls: ``type`` or :obj:`None <python:None>`

    :param method: The method to search for in the MRO. Defaults to
      ``'_to_untrimmed_dict'``.
    :type method: :class:`str <python:str>`

    :returns: List of classes that have ``method`` that occur *after* ``in_cls`` in
      the MRO for ``cls``.
    :rtype: :class:`list <python:list>` of ``type`` objects
    """
    mro = [x for x in cls.mro()
           if hasattr(x, method) and x.__name__ != 'HighchartsMeta']
    if in_cls is None:
        return mro[1:]
    else:
        index = mro.index(in_cls)
        return mro[(index + 1):]


def mro__to_untrimmed_dict(obj, in_cls = None):
    """Traverse the ancestor classes of ``obj`` and execute their ``_to_untrimmed_dict()``
    methods.

    :param obj: The object to be traversed.
    :type obj: :class:`HighchartsMeta`

    :param in_cls: The class from which ``mro__to_untrimmed_dict()`` was called.
    :type in_cls: ``type`` or :obj:`None <python:None>`

    :returns: Collection of untrimmed :class:`dict <python:dict>` representations in the
      same order as the MRO.
    :rtype: :class:`list <python:list>` of :class:`dict <python:dict>`

    for each class in the MRO, execute _to_untrimmed_dict()
    do not repeat for each class
    """
    cls = obj.__class__
    remaining_mro = get_remaining_mro(cls,
                                      in_cls = in_cls,
                                      method = '_to_untrimmed_dict')

    ancestor_dicts = []
    for x in remaining_mro:
        if hasattr(x, '_to_untrimmed_dict') and x != cls:
            ancestor_dicts.append(x._to_untrimmed_dict(obj,
                                                       in_cls = x))

    consolidated = {}
    for item in ancestor_dicts:
        for key in item:
            consolidated[key] = item[key]

    return consolidated


def validate_color(value):
    """Validate that ``value`` is either a :class:`Gradient`, :class:`Pattern`, or a
    :class:`str <python:str>`.

    :param value: The value to validate.

    :returns: The validated value.
    :rtype: :class:`str <python:str>`, :class:`Gradient`, :class:`Pattern``, or
      :obj:`None <python:None>`
    """
    from highcharts_core.utility_classes.gradients import Gradient
    from highcharts_core.utility_classes.patterns import Pattern

    if not value:
        return None
    elif value.__class__.__name__ == 'EnforcedNullType':
        return value
    elif isinstance(value, (Gradient, Pattern)):
        return value
    elif isinstance(value, (dict, str)) and ('linearGradient' in value or
                                             'radialGradient' in value):
        try:
            value = Gradient.from_json(value)
        except (TypeError, ValueError):
            if isinstance(value, dict):
                value = Gradient.from_dict(value)
            else:
                value = validators.string(value)
    elif isinstance(value, dict) and ('linear_gradient' in value or
                                      'radial_gradient' in value):
        value = Gradient(**value)
    elif isinstance(value, (dict, str)) and ('patternOptions' in value or
                                             'pattern' in value):
        try:
            value = Pattern.from_json(value)
        except (TypeError, ValueError):
            if isinstance(value, dict):
                value = Pattern.from_dict(value)
            else:
                value = validators.string(value)
    elif isinstance(value, dict) and 'pattern_options' in value:
        value = Pattern(**value)
    elif isinstance(value, str):
        value = validators.string(value)
    else:
        raise errors.HighchartsValueError(f'Unable to resolve value to a string, '
                                          f'Gradient, or Pattern. Value received '
                                          f'was: {value}')

    return value


def to_camelCase(snake_case):
    """Convert ``snake_case`` to ``camelCase``.

    :param snake_case: A :class:`str <python:str>` which is likely to contain
      ``snake_case``.
    :type snake_case: :class:`str <python:str>`

    :returns: A ``camelCase`` representation of ``snake_case``.
    :rtype: :class:`str <python:str>`
    """
    snake_case = validators.string(snake_case)

    if '_' not in snake_case:
        return snake_case

    if 'url' in snake_case:
        snake_case = snake_case.replace('url', 'URL')
    elif 'utc' in snake_case:
        snake_case = snake_case.replace('utc', 'UTC')
    elif '_csv' in snake_case:
        snake_case = snake_case.replace('csv', 'CSV')
    elif '_jpeg' in snake_case:
        snake_case = snake_case.replace('jpeg', 'JPEG')
    elif '_pdf' in snake_case:
        snake_case = snake_case.replace('pdf', 'PDF')
    elif '_png' in snake_case:
        snake_case = snake_case.replace('png', 'PNG')
    elif '_svg' in snake_case:
        snake_case = snake_case.replace('svg', 'SVG')
    elif '_xls' in snake_case:
        snake_case = snake_case.replace('xls', 'XLS')
    elif '_atr' in snake_case:
        snake_case = snake_case.replace('atr', 'ATR')
    elif '_hlc' in snake_case:
        snake_case = snake_case.replace('hlc', 'HLC')
    elif '_ohlc' in snake_case:
        snake_case = snake_case.replace('ohlc', 'OHLC')
    elif '_xy' in snake_case:
        snake_case = snake_case.replace('xy', 'XY')
    elif snake_case.endswith('_x'):
        snake_case = snake_case.replace('_x', '_X')
    elif snake_case.endswith('_y'):
        snake_case = snake_case.replace('_y', '_Y')
    elif snake_case.endswith('_id'):
        snake_case = snake_case.replace('_id', '_ID')
    elif snake_case == 'drillup_text':
        snake_case = 'drillUpText'
    elif snake_case == 'drillup_button':
        snake_case = 'drillUpButton'
    elif snake_case == 'thousands_separator':
        snake_case = 'thousandsSep'
    elif snake_case == 'measure_xy':
        snake_case = 'measureXY'
    elif snake_case == 'use_gpu_translations':
        snake_case = 'useGPUTranslations'
    elif snake_case == 'label_rank':
        snake_case = 'labelrank'
    elif '_di_line' in snake_case:
        snake_case = snake_case.replace('_di_line', '_DILine')

    camel_case = ''
    previous_character = ''
    for character in snake_case:
        if character != '_' and previous_character != '_':
            camel_case += character
            previous_character = character
        elif character == '_':
            previous_character = character
        elif character != '_' and previous_character == '_':
            camel_case += character.upper()
            previous_character = character

    return camel_case


def parse_csv(csv_data,
              has_header_row = True,
              delimiter = ',',
              null_text = 'None',
              wrapper_character = "'",
              wrap_all_strings = False,
              double_wrapper_character_when_nested = False,
              escape_character = "\\",
              line_terminator = '\r\n'):
    """Parse ``csv_data`` to return a list of :class:`dict <python:dict>` objects, one
    for each record.

    :param csv_data: The CSV record expressed as a :class:`str <python:str>`
    :type csv_data: :class:`str <python:str>`

    :param delimiter: The delimiter used between columns. Defaults to ``,``.
    :type delimiter: :class:`str <python:str>`

    :param wrapper_character: The string used to wrap string values when
      wrapping is applied. Defaults to ``'``.
    :type wrapper_character: :class:`str <python:str>`

    :param null_text: The string used to indicate an empty value if empty
      values are wrapped. Defaults to `None`.
    :type null_text: :class:`str <python:str>`

    :returns: Collection of column names (or numerical keys) and CSV records as
      :class:`dict <python:dict>` values
    :rtype: :class:`tuple <python:tuple>` of a :class:`list <python:list>` of column names
      and :class:`list <python:list>` of :class:`dict <python:dict>`
    """
    if not csv_data:
        return [], []

    if isinstance(csv_data, str):
        csv_data = csv_data.split(line_terminator)

    if not wrapper_character:
        wrapper_character = "'"

    if wrap_all_strings:
        quoting = csv.QUOTE_NONNUMERIC
    else:
        quoting = csv.QUOTE_MINIMAL

    if 'highcharts' in csv.list_dialects():
        csv.unregister_dialect('highcharts')

    csv.register_dialect('highcharts',
                         delimiter = delimiter,
                         doublequote = double_wrapper_character_when_nested,
                         escapechar = escape_character,
                         quotechar = wrapper_character,
                         quoting = quoting,
                         lineterminator = line_terminator)

    if has_header_row:
        csv_reader = csv.DictReader(csv_data,
                                    dialect = 'highcharts',
                                    restkey = None,
                                    restval = None)
        records_as_dicts = [x for x in csv_reader]
        columns = csv_reader.fieldnames
    else:
        csv_reader = csv.reader(csv_data,
                                dialect = 'highcharts')
        records_as_dicts = []
        columns = []
        for row in csv_reader:
            record_as_dict = {}
            column_counter = 0
            for column in row:
                record_as_dict[column_counter] = column
                columns.append(column_counter)
                column_counter += 1

            records_as_dicts.append(record_as_dict)

    return columns, records_as_dicts


def jupyter_add_script(url, is_last = False):
    """Generates the JavaScript code Promise which adds a <script/> tag to the Jupyter 
    Lab environment.
    
    :param url: The URL to use for the script's source.
    :type url: :class:`str <python:str>`
    
    :param is_last: Whether the URL is the last of the promises.
    :type is_last: :class:`bool <python:bool>`
    
    :returns: The JavaScript code for adding the script.
    :rtype: :class:`str <python:str>`
    """
    url = validators.url(url)
    if url.endswith('.css'):
        return jupyter_add_link(url, is_last = is_last)
    
    js_str = """"""
    js_str += """new Promise(function(resolve, reject) {\n"""
    js_str += f"""  var existing_tags = document.querySelectorAll("script[src='{url}']");"""
    js_str += """  if (existing_tags.length == 0) {
        var script = document.createElement("script");
        script.onload = resolve;
        script.onerror = reject;"""
    js_str += f"""        script.src = '{url}';"""
    js_str += """        document.head.appendChild(script);
    };
})"""

    return js_str


def jupyter_add_link(url, is_last = False):
    """Generates the JavaScript code Promise which adds a <link/> tag to the Jupyter 
    Lab environment.
    
    :param url: The URL to use for the link's source.
    :type url: :class:`str <python:str>`
    
    :param is_last: Whether the URL is the last of the promises.
    :type is_last: :class:`bool <python:bool>`
    
    :returns: The JavaScript code for adding the link.
    :rtype: :class:`str <python:str>`
    """
    url = validators.url(url)
    
    js_str = ''
    js_str += """new Promise(function(resolve, reject) {\n"""
    js_str += f"""  var existing_tags = document.querySelectorAll("link[href='{url}']");"""
    js_str += """  if (existing_tags.length == 0) {
        var link = document.createElement("link");
        link.onload = resolve;
        link.onerror = reject;"""
    js_str += f"""        link.href = '{url}';"""
    js_str += f"""        link.rel = 'stylesheet';"""
    js_str += f"""        link.type = 'text/css';"""
    js_str += """        document.head.appendChild(link);
    };
})"""

    return js_str


def get_retryHighcharts():
    """Retrieve the ``retryHighcharts()`` JavaScript function.
    
    :returns: The JavaScript code of the ``retryHighcharts()`` JavaScript function.
    :rtype: :class:`str <python:str>`
    """
    js_str = """function retryHighcharts(fn, container = 'highcharts_target_div', retries = 5, retriesLeft = 5, 
        interval = 1000) {
            return new Promise((resolve, reject) => {
            try {
                fn()
                return resolve();
            } catch (err) {
                if ((err instanceof ReferenceError) || (err instanceof TypeError) || (err.message.includes('#17'))) {
                    if (retriesLeft === 0) {
                        var target_div = document.getElementById(container);
                        if (target_div) {
                            var timeElapsed = (retries * interval) / 1000;
                            var errorMessage = "Something went wrong with the Highcharts.js script. It should have been automatically loaded, but it did not load for over " + timeElapsed + " seconds. Check your internet connection, and then if the problem persists please reach out for support. (You can also check your browser's console log for more details.)<br/><br/>Detailed Error Message:<br/>" + err.message;
                            var errorHTML = "<p>" + errorMessage + "</p>";
                            target_div.innerHTML = errorMessage;
                            console.log(errorMessage);
                            console.error(err);
                        }
                        return reject();
                    }

                    setTimeout(() => {
                        retryHighcharts(fn, container, retries, retriesLeft - 1, interval).then(resolve).catch(reject);
                    }, interval);
                } else if ((err instanceof Error) && (err.message.includes('#13'))) {
                    var errorMessage = "It looks like the container specified \'" + container + "\' was not created successfully. Please check your browser\'s console log for more details.";
                    console.error(errorMessage);
                    console.error(err);
                    
                    return reject();
                } else {
                    throw err;
                }
            }
        });
    };"""
    
    return js_str


def prep_js_for_jupyter(js_str,
                        container = 'highcharts_target_div',
                        random_slug = None,
                        retries = 5,
                        interval = 1000):
    """Remove the JavaScript event listeners from the code in ``js_str`` and prepare the
    JavaScript code for rending in an IPython context.
    
    :param js_str: The JavaScript code from which the event listeners should be stripped.
    :type js_str: :class:`str <python:str>`
    
    :param container: The DIV where the Highcharts visualization is to be rendered. Defaults to
      ``'highcharts_target_div'``.
    :type container: :class:`str <python:str>`
    
    :param random_slug: The random sequence of characters to append to the container/function name to ensure uniqueness.
        Defaults to :obj:`None <python:None>`
    :type random_slug: :class:`str <python:str>` or :obj:`None <python:None>`

    :param retries: The number of times to retry the rendering. Defaults to 3.
    :type retries: :class:`int <python:int>`
    
    :param interval: The number of milliseconds to wait between retries. Defaults to 1000 (1 second).
    :type interval: :class:`int <python:int>`
    
    :returns: The JavaScript code having removed the non-Jupyter compliant JS code.
    :rtype: :class:`str <python:str>`
    """
    js_str = js_str.replace(
        """document.addEventListener('DOMContentLoaded', function() {""", '')
    js_str = js_str.replace('renderTo = ', '')
    js_str = js_str.replace(',\noptions = ', ',\n')
    js_str = js_str[:-3]

    if random_slug:
        function_str = f"""function insertChart_{random_slug}() """
    else:
        function_str = """function insertChart() """
    function_str += """{\n"""
    function_str += js_str
    function_str += """\n};\n"""
    if random_slug:
        function_str += f"""retryHighcharts(insertChart_{random_slug}, '{container}', {retries}, {retries}, {interval});"""
    else:
        function_str += f"""retryHighcharts(insertChart, '{container}', {retries}, {retries}, {interval});"""

    return function_str



from typing import Optional

from validator_collection import validators

from highcharts_core import constants
from highcharts_core.decorators import class_sensitive
from highcharts_core.metaclasses import HighchartsMeta
from highcharts_core.global_options.language.accessibility.announce_new_data import AnnounceNewDataLanguageOptions
from highcharts_core.global_options.language.accessibility.axis import AxisLanguageOptions
from highcharts_core.global_options.language.accessibility.chart_types import ChartTypesLanguageOptions
from highcharts_core.global_options.language.accessibility.exporting import ExportingLanguageOptions
from highcharts_core.global_options.language.accessibility.legend import LegendLanguageOptions
from highcharts_core.global_options.language.accessibility.range_selector import RangeSelectorLanguageOptions
from highcharts_core.global_options.language.accessibility.screen_reader_section import ScreenReaderSectionLanguageOptions
from highcharts_core.global_options.language.accessibility.series import SeriesLanguageOptions, SeriesTypeDescriptions
from highcharts_core.global_options.language.accessibility.sonification import SonificationLanguageOptions
from highcharts_core.global_options.language.accessibility.table import TableLanguageOptions
from highcharts_core.global_options.language.accessibility.zoom import ZoomLanguageOptions


class AccessibilityLanguageOptions(HighchartsMeta):
    """Configuration of accessibility strings in the chart.

    .. note::

      Requires the
      `accessibility module <https://code.highcharts.com/modules/accessibility.js>`_
      to be loaded.

      For a description of the module and information on its features, see
      `Accessibility <https://www.highcharts.com/docs/chart-concepts/accessibility>`_.

    """

    def __init__(self, **kwargs):
        self._announce_new_data = None
        self._axis = None
        self._chart_container_label = None
        self._chart_types = None
        self._credits = None
        self._default_chart_title = None
        self._drillup_button = None
        self._exporting = None
        self._graphic_container_label = None
        self._legend = None
        self._range_selector = None
        self._screen_reader_section = None
        self._series = None
        self._series_type_descriptions = None
        self._sonification = None
        self._svg_container_label = None
        self._svg_container_title = None
        self._table = None
        self._thousands_separator = None
        self._zoom = None

        self.announce_new_data = kwargs.get('announce_new_data', None)
        self.axis = kwargs.get('axis', None)
        self.chart_container_label = kwargs.get('chart_container_label', None)
        self.chart_types = kwargs.get('chart_types', None)
        self.credits = kwargs.get('credits', None)
        self.default_chart_title = kwargs.get('default_chart_title', None)
        self.drillup_button = kwargs.get('drillup_button', None)
        self.exporting = kwargs.get('exporting', None)
        self.graphic_container_label = kwargs.get('graphic_container_label', None)
        self.legend = kwargs.get('legend', None)
        self.range_selector = kwargs.get('range_selector', None)
        self.screen_reader_section = kwargs.get('screen_reader_section', None)
        self.series = kwargs.get('series', None)
        self.series_type_description = kwargs.get('series_type_description', None)
        self.sonification = kwargs.get('sonification', None)
        self.svg_container_label = kwargs.get('svg_container_label', None)
        self.svg_container_title = kwargs.get('svg_container_title', None)
        self.table = kwargs.get('table', None)
        self.thousands_separator = kwargs.get('thousands_separator', None)
        self.zoom = kwargs.get('zoom', None)

    @property
    def announce_new_data(self) -> Optional[AnnounceNewDataLanguageOptions]:
        """Default announcement for new data in charts.

        .. note::

          If (JavaScript) ``addPoint()`` or ``addSeries()`` is used, and only one
          series/point is added, the
          :meth:`AnnounceNewDataLanguageOptions.new_point_announce` and
          :meth:`AnnounceNewdata.new_series_announce` strings are used.

          The ``...single`` versions will be used if there is only one chart on the page,
          and the ``...multiple`` versions will be used if there are multiple charts on the
          page. For all other new data events, the
          :meth:`AnnounceNewData.new_data_announce` string will be used.

        :rtype: :class:`AnnounceNewDataLanguageOptions` or :obj:`None <python:None>`
        """
        return self._announce_new_data

    @announce_new_data.setter
    @class_sensitive(AnnounceNewDataLanguageOptions)
    def announce_new_data(self, value):
        self._announce_new_data = value

    @property
    def axis(self) -> Optional[AxisLanguageOptions]:
        """Axis description strings.

        :rtype: :class:`AxisLanguageOptions` or :obj:`None <python:None>`
        """
        return self._axis

    @axis.setter
    @class_sensitive(AxisLanguageOptions)
    def axis(self, value):
        self._axis = value

    @property
    def chart_container_label(self) -> Optional[str]:
        """Label applied to the chart container. Defaults to
        ``'{title}. Highcharts interactive chart.'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._chart_container_label

    @chart_container_label.setter
    def chart_container_label(self, value):
        self._chart_container_label = validators.string(value, allow_empty = True)

    @property
    def chart_types(self) -> Optional[ChartTypesLanguageOptions]:
        """Chart type description strings.

        .. note::

          This is added to the chart information region.

          If there is only a single series type used in the chart, we use the format
          string for the series type, or default if missing. There is one format string
          for cases where there is only a single series in the chart, and one for multiple
          series of the same type.

        :rtype: :class:`ChartTypesLanguageOptions` or :obj:`None <python:None>`
        """
        return self._chart_types

    @chart_types.setter
    @class_sensitive(ChartTypesLanguageOptions)
    def chart_types(self, value):
        self._chart_types = value

    @property
    def credits(self) -> Optional[str]:
        """Content of the credits section. Defaults to:
        ``'Chart credits: {creditsStr}'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._credits

    @credits.setter
    def credits(self, value):
        self._credits = validators.string(value, allow_empty = True)

    @property
    def default_chart_title(self) -> Optional[str]:
        """The default title applied ot the chart. Defaults to
        ``'Chart'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._default_chart_title

    @default_chart_title.setter
    def default_chart_title(self, value):
        self._default_chart_title = validators.string(value, allow_empty = True)

    @property
    def drillup_button(self) -> Optional[str]:
        """String for the drill-up button. Defaults to
        ``'{buttonText}'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._drillup_button

    @drillup_button.setter
    def drillup_button(self, value):
        self._drillup_button = validators.string(value, allow_empty = True)

    @property
    def exporting(self) -> Optional[ExportingLanguageOptions]:
        """Exporting menu format strings for use in the accessibility module.

        :rtype: :class:`ExportingLanguageOptions` or :obj:`None <python:None>`
        """
        return self._exporting

    @exporting.setter
    @class_sensitive(ExportingLanguageOptions)
    def exporting(self, value):
        self._exporting = value

    @property
    def graphic_container_label(self) -> Optional[str]:
        """Set a label on the container wrapping the SVG. Defaults to
        ``''`` (an empty
        string).

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._graphic_container_label

    @graphic_container_label.setter
    def graphic_container_label(self, value):
        self._graphic_container_label = validators.string(value, allow_empty = True)

    @property
    def legend(self) -> Optional[LegendLanguageOptions]:
        """Language options for the legend when used in accessibility mode.

        :rtype: :class:`LegendLanguageOptions` or :obj:`None <python:None>`
        """
        return self._legend

    @legend.setter
    @class_sensitive(LegendLanguageOptions)
    def legend(self, value):
        self._legend = value

    @property
    def range_selector(self) -> Optional[RangeSelectorLanguageOptions]:
        """Language options for range selectors when used in accessibility mode.

        :rtype: :class:`RangeSelectorLanguageOptions` or :obj:`None <python:None>`
        """
        return self._range_selector

    @range_selector.setter
    @class_sensitive(RangeSelectorLanguageOptions)
    def range_selector(self, value):
        self._range_selector = value

    @property
    def screen_reader_section(self) -> Optional[ScreenReaderSectionLanguageOptions]:
        """Language options for the screen reader information sections added before and
        after the chart when used in accessibility mode.

        :rtype: :class:`ScreenReaderSectionLanguageOptions` or :obj:`None <python:None>`
        """
        return self._screen_reader_section

    @screen_reader_section.setter
    @class_sensitive(ScreenReaderSectionLanguageOptions)
    def screen_reader_section(self, value):
        self._screen_reader_section = value

    @property
    def series(self) -> Optional[SeriesLanguageOptions]:
        """Language configuration for different series types.

        .. hint::

          For more dynamic control over the series element descriptions, see
          :meth:`Accessibility.series_description_formatter`.

        :rtype: :class:`SeriesLanguageOptions` or :obj:`None <python:None>`
        """
        return self._series

    @series.setter
    @class_sensitive(SeriesLanguageOptions)
    def series(self, value):
        self._series = value

    @property
    def series_type_descriptions(self) -> Optional[SeriesTypeDescriptions]:
        """Descriptions of lesser known series types. The relevant description is added to
        the screen reader information region when these series types are used.

        :rtype: :class:`SeriesTypeDescriptions` or :obj:`None <python:None>`
        """
        return self._series_type_descriptions

    @series_type_descriptions.setter
    @class_sensitive(SeriesTypeDescriptions)
    def series_type_descriptions(self, value):
        self._series_type_descriptions = value

    @property
    def sonification(self) -> Optional[SonificationLanguageOptions]:
        """Language options for the sonification functionality when used in accessibility
        mode.

        :rtype: :class:`SonificationLanguageOptions` or :obj:`None <python:None>`
        """
        return self._sonification

    @sonification.setter
    @class_sensitive(SonificationLanguageOptions)
    def sonification(self, value):
        self._sonification = value

    @property
    def svg_container_label(self) -> Optional[str]:
        """Set a label on the container wrapping the SVG. Defaults to
        ``'Interactive chart'``.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._svg_container_label

    @svg_container_label.setter
    def svg_container_label(self, value):
        self._svg_container_label = validators.string(value, allow_empty = True)

    @property
    def svg_container_title(self) -> Optional[str]:
        """Title element text for the chart SVG element. Defaults to
        ``''`` (an empty
        string).

        .. note::

          Leave this empty to disable adding the title element. Browsers will display this
          content when hovering over elements in the chart. Assistive technology may use
          this element to label the chart.

        :rtype: :class:`str <python:str>` or :obj:`None <python:None>`
        """
        return self._svg_container_title

    @svg_container_title.setter
    def svg_container_title(self, value):
        self._svg_container_title = validators.string(value, allow_empty = True)

    @property
    def table(self) -> Optional[TableLanguageOptions]:
        """Language options for the table functionality when used in accessibility
        mode.

        :rtype: :class:`TableLanguageOptions` or :obj:`None <python:None>`
        """
        return self._table

    @table.setter
    @class_sensitive(TableLanguageOptions)
    def table(self, value):
        self._table = value

    @property
    def thousands_separator(self) -> Optional[str | constants.EnforcedNullType]:
        """Thousands separator to use when formatting numbers for screen readers.
        Defaults to ``','``.

        Set to :obj:`None <python:None>` or to an :class:`EnforcedNullType` to use the
        separator defined in :meth:`Language.thousands_separator`.

        .. note::

          Note that many screen readers will not handle an empty space as a thousands
          separator, and will consider "11 700" as two numbers.

        :rtype: :class:`str <python:str>` or :class:`EnforcedNullType` or
          :obj:`None <python:None>`
        """
        return self._thousands_separator

    @thousands_separator.setter
    def thousands_separator(self, value):
        if not value:
            self._thousands_separator = None
        elif isinstance(value, constants.EnforcedNullType):
            self._thousands_separator = constants.EnforcedNull
        else:
            self._thousands_separator = validators.string(value, allow_empty = True)

    @property
    def zoom(self) -> Optional[ZoomLanguageOptions]:
        """Language options for the zoom functionality when used in accessibility
        mode.

        :rtype: :class:`ZoomLanguageOptions` or :obj:`None <python:None>`
        """
        return self._zoom

    @zoom.setter
    @class_sensitive(ZoomLanguageOptions)
    def zoom(self, value):
        self._zoom = value

    @classmethod
    def _get_kwargs_from_dict(cls, as_dict):
        kwargs = {
            'announce_new_data': as_dict.get('announceNewData', None),
            'axis': as_dict.get('axis', None),
            'chart_container_label': as_dict.get('chartContainerLabel', None),
            'chart_types': as_dict.get('chartTypes', None),
            'credits': as_dict.get('credits', None),
            'default_chart_title': as_dict.get('defaultChartTitle', None),
            'drillup_button': as_dict.get('drillUpButton', None),
            'exporting': as_dict.get('exporting', None),
            'graphic_container_label': as_dict.get('graphicContainerLabel', None),
            'legend': as_dict.get('legend', None),
            'range_selector': as_dict.get('rangeSelector', None),
            'screen_reader_section': as_dict.get('screenReaderSection', None),
            'series': as_dict.get('series', None),
            'series_type_description': as_dict.get('seriesTypeDescription', None),
            'sonification': as_dict.get('sonification', None),
            'svg_container_label': as_dict.get('svgContainerLabel', None),
            'svg_container_title': as_dict.get('svgContainerTitle', None),
            'table': as_dict.get('table', None),
            'thousands_separator': as_dict.get('thousandsSep', None),
            'zoom': as_dict.get('zoom', None)
        }

        return kwargs

    def _to_untrimmed_dict(self, in_cls = None) -> dict:
        untrimmed = {
            'announceNewData': self.announce_new_data,
            'axis': self.axis,
            'chartContainerLabel': self.chart_container_label,
            'chartTypes': self.chart_types,
            'credits': self.credits,
            'defaultChartTitle': self.default_chart_title,
            'drillUpButton': self.drillup_button,
            'exporting': self.exporting,
            'graphicContainerLabel': self.graphic_container_label,
            'legend': self.legend,
            'rangeSelector': self.range_selector,
            'screenReaderSection': self.screen_reader_section,
            'series': self.series,
            'seriesTypeDescription': self.series_type_description,
            'sonification': self.sonification,
            'svgContainerLabel': self.svg_container_label,
            'svgContainerTitle': self.svg_container_title,
            'table': self.table,
            'thousandsSep': self.thousands_separator,
            'zoom': self.zoom
        }

        return untrimmed


__all__ = [
    'AccessibilityLanguageOptions',
    'AnnounceNewDataLanguageOptions',
    'AxisLanguageOptions',
    'ChartTypesLanguageOptions',
    'ExportingLanguageOptions',
    'LegendLanguageOptions',
    'RangeSelectorLanguageOptions',
    'ScreenReaderSectionLanguageOptions',
    'SeriesLanguageOptions',
    'SeriesTypeDescriptions',
    'SonificationLanguageOptions',
    'TableLanguageOptions',
    'ZoomLanguageOptions'
]
